@@ -20,6 +20,8 @@
 
 #include "config.h"
 
+#include "conn-avatars.h"
+
 #include <string.h>
 
 #include <loudmouth/loudmouth.h>
@@ -34,8 +36,6 @@
 #include "vcard-manager.h"
 #include "util.h"
 
-#include "conn-avatars.h"
-
 #define DEBUG_FLAG GABBLE_DEBUG_CONNECTION
 
 #include "debug.h"
@@ -133,6 +133,9 @@ gabble_connection_get_avatar_requirements (TpSvcConnectionInterfaceAvatars *ifac
   static const char *mimetypes[] = {
       "image/png", "image/jpeg", "image/gif", NULL };
 
+  TP_BASE_CONNECTION_ERROR_IF_NOT_CONNECTED (TP_BASE_CONNECTION (iface),
+      context);
+
   /* Jabber has no min/max width/height or max size, but XEP-0153 says
    * you SHOULD use 32-96px either way, and no more than 8K of data */
 
@@ -140,6 +143,7 @@ gabble_connection_get_avatar_requirements (TpSvcConnectionInterfaceAvatars *ifac
       context, mimetypes, 32, 32, 96, 96, 8192);
 }
 
+/* begin deprecated code */
 
 typedef struct {
   DBusGMethodInvocation *invocation;
@@ -148,7 +152,6 @@ typedef struct {
   gulong signal_conn;
 } GetAvatarTokensContext;
 
-
 static void
 _got_self_avatar_for_get_avatar_tokens (GObject *obj,
                                         gchar *sha1,
@@ -158,7 +161,7 @@ _got_self_avatar_for_get_avatar_tokens (GObject *obj,
 
   g_signal_handler_disconnect (obj, context->signal_conn);
   g_free (context->ret[context->my_index]);
-  context->ret[context->my_index] = g_strdup(sha1);
+  context->ret[context->my_index] = g_strdup (sha1);
 
   /* FIXME: I'm not entirely sure why gcc warns without this cast from
    * (gchar **) to (const gchar **) */
@@ -190,10 +193,13 @@ gabble_connection_get_avatar_tokens (TpSvcConnectionInterfaceAvatars *iface,
   gboolean have_self_avatar;
   guint i, my_index = 0;
   gchar **ret;
-  GError *err;
+  GError *err = NULL;
+  TpHandleRepoIface *contact_handles = tp_base_connection_get_handles (base,
+      TP_HANDLE_TYPE_CONTACT);
 
-  if (!tp_handles_are_valid (base->handles[TP_HANDLE_TYPE_CONTACT],
-        contacts, FALSE, &err))
+  TP_BASE_CONNECTION_ERROR_IF_NOT_CONNECTED (base, invocation);
+
+  if (!tp_handles_are_valid (contact_handles, contacts, FALSE, &err))
     {
       dbus_g_method_return_error (invocation, err);
       g_error_free (err);
@@ -262,6 +268,195 @@ gabble_connection_get_avatar_tokens (TpSvcConnectionInterfaceAvatars *iface,
   g_strfreev (ret);
 }
 
+/* end deprecated code */
+
+typedef struct {
+  GabbleConnection *conn;
+  DBusGMethodInvocation *invocation;
+  GHashTable *ret;
+  gulong signal_conn;
+} GetKnownAvatarTokensContext;
+
+static void
+_got_self_avatar_for_get_known_avatar_tokens (GObject *obj,
+                                              gchar *sha1,
+                                              gpointer user_data)
+{
+  GetKnownAvatarTokensContext *context =
+      (GetKnownAvatarTokensContext *) user_data;
+  TpBaseConnection *base = (TpBaseConnection *) context->conn;
+
+  g_signal_handler_disconnect (obj, context->signal_conn);
+
+  g_hash_table_insert(context->ret, GUINT_TO_POINTER (base->self_handle),
+      g_strdup(sha1));
+
+  tp_svc_connection_interface_avatars_return_from_get_known_avatar_tokens (
+      context->invocation, context->ret);
+  g_hash_table_destroy (context->ret);
+
+  g_slice_free (GetKnownAvatarTokensContext, context);
+}
+
+
+/**
+ * gabble_connection_get_avatar_tokens
+ *
+ * Implements D-Bus method GetAvatarTokens
+ * on interface org.freedesktop.Telepathy.Connection.Interface.Avatars
+ *
+ * @context: The D-Bus invocation context to use to return values
+ *           or throw an error.
+ */
+static void
+gabble_connection_get_known_avatar_tokens (TpSvcConnectionInterfaceAvatars *iface,
+                                           const GArray *contacts,
+                                           DBusGMethodInvocation *invocation)
+{
+  GabbleConnection *self = GABBLE_CONNECTION (iface);
+  TpBaseConnection *base = (TpBaseConnection *)self;
+  gboolean wait_for_self_avatar = FALSE;
+  gboolean have_self_avatar;
+  guint i;
+  GHashTable *ret;
+  GError *err = NULL;
+  TpHandleRepoIface *contact_handles = tp_base_connection_get_handles (base,
+      TP_HANDLE_TYPE_CONTACT);
+
+  TP_BASE_CONNECTION_ERROR_IF_NOT_CONNECTED (base, invocation);
+
+  if (!tp_handles_are_valid (contact_handles, contacts, FALSE, &err))
+    {
+      dbus_g_method_return_error (invocation, err);
+      g_error_free (err);
+      return;
+    }
+
+  g_object_get (self->vcard_manager,
+      "have-self-avatar", &have_self_avatar,
+      NULL);
+
+  ret = g_hash_table_new_full (NULL, NULL, NULL, g_free);
+
+  /* TODO: always call the callback so we can defer presence lookups until
+   * we return the method, then we don't need to strdup the strings we're
+   * returning. */
+
+  for (i = 0; i < contacts->len; i++)
+    {
+      TpHandle handle;
+      GabblePresence *presence = NULL;
+
+      handle = g_array_index (contacts, TpHandle, i);
+
+      if (base->self_handle == handle)
+        {
+          if (have_self_avatar)
+            {
+              presence = self->self_presence;
+            }
+          else
+            {
+              wait_for_self_avatar = TRUE;
+            }
+        }
+      else
+        {
+          presence = gabble_presence_cache_get (self->presence_cache, handle);
+        }
+
+      if (NULL != presence)
+        {
+          if (NULL != presence->avatar_sha1)
+              g_hash_table_insert (ret, GUINT_TO_POINTER (handle),
+                  g_strdup (presence->avatar_sha1));
+          else
+              g_hash_table_insert (ret, GUINT_TO_POINTER (handle), g_strdup (""));
+        }
+    }
+
+  if (wait_for_self_avatar)
+    {
+      GetKnownAvatarTokensContext *context = g_slice_new (GetKnownAvatarTokensContext);
+
+      context->invocation = invocation;
+      context->ret = ret;
+      context->signal_conn = g_signal_connect (self->vcard_manager,
+          "got-self-initial-avatar",
+          G_CALLBACK (_got_self_avatar_for_get_known_avatar_tokens),
+          context);
+
+      return;
+    }
+
+  tp_svc_connection_interface_avatars_return_from_get_known_avatar_tokens (
+      invocation, ret);
+
+  g_hash_table_destroy (ret);
+}
+
+
+
+static gboolean
+parse_avatar (LmMessageNode *vcard,
+              const gchar **mime_type,
+              GString **avatar,
+              GError **error)
+{
+  LmMessageNode *photo_node;
+  LmMessageNode *type_node;
+  LmMessageNode *binval_node;
+  const gchar *binval_value;
+
+  photo_node = lm_message_node_get_child (vcard, "PHOTO");
+
+  if (NULL == photo_node)
+    {
+      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+        "contact vCard has no photo");
+      return FALSE;
+    }
+
+  type_node = lm_message_node_get_child (photo_node, "TYPE");
+
+  if (NULL != type_node)
+    {
+      *mime_type = lm_message_node_get_value (type_node);
+    }
+  else
+    {
+      *mime_type = "";
+    }
+
+  binval_node = lm_message_node_get_child (photo_node, "BINVAL");
+
+  if (NULL == binval_node)
+    {
+      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+        "contact avatar is missing binval node");
+      return FALSE;
+    }
+
+  binval_value = lm_message_node_get_value (binval_node);
+
+  if (NULL == binval_value)
+    {
+      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+        "contact avatar is missing binval content");
+      return FALSE;
+    }
+
+  *avatar = base64_decode (binval_value);
+
+  if (NULL == *avatar)
+    {
+      g_set_error (error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
+        "failed to decode avatar from base64");
+      return FALSE;
+    }
+
+  return TRUE;
+}
 
 static void
 _request_avatar_cb (GabbleVCardManager *self,
@@ -274,11 +469,10 @@ _request_avatar_cb (GabbleVCardManager *self,
   DBusGMethodInvocation *context = (DBusGMethodInvocation *) user_data;
   GabbleConnection *conn;
   TpBaseConnection *base;
-  LmMessageNode *photo_node, *type_node, *binval_node;
-  const gchar *mime_type;
+  const gchar *mime_type = NULL;
   GArray *arr;
   GError *error = NULL;
-  GString *avatar;
+  GString *avatar = NULL;
   GabblePresence *presence;
 
   g_object_get (self, "connection", &conn, NULL);
@@ -290,45 +484,8 @@ _request_avatar_cb (GabbleVCardManager *self,
       goto out;
     }
 
-  photo_node = lm_message_node_get_child (vcard, "PHOTO");
-
-  if (NULL == photo_node)
+  if (!parse_avatar (vcard, &mime_type, &avatar, &error))
     {
-      g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
-        "contact vCard has no photo");
-      dbus_g_method_return_error (context, error);
-      g_error_free (error);
-      goto out;
-    }
-
-  type_node = lm_message_node_get_child (photo_node, "TYPE");
-
-  if (NULL == type_node)
-    {
-      g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
-        "contact avatar is missing type node");
-      dbus_g_method_return_error (context, error);
-      g_error_free (error);
-      goto out;
-    }
-
-  binval_node = lm_message_node_get_child (photo_node, "BINVAL");
-
-  if (NULL == binval_node)
-    {
-      g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
-        "contact avatar is missing binval node");
-      dbus_g_method_return_error (context, error);
-      g_error_free (error);
-      goto out;
-    }
-
-  avatar = base64_decode (lm_message_node_get_value (binval_node));
-
-  if (NULL == avatar)
-    {
-      g_set_error (&error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE,
-        "failed to decode avatar from base64");
       dbus_g_method_return_error (context, error);
       g_error_free (error);
       goto out;
@@ -379,7 +536,6 @@ _request_avatar_cb (GabbleVCardManager *self,
       g_free (sha1);
     }
 
-  mime_type = lm_message_node_get_value (type_node);
   arr = g_array_new (FALSE, FALSE, sizeof (gchar));
   g_array_append_vals (arr, avatar->str, avatar->len);
   tp_svc_connection_interface_avatars_return_from_request_avatar (
@@ -387,6 +543,9 @@ _request_avatar_cb (GabbleVCardManager *self,
   g_array_free (arr, TRUE);
 
 out:
+  if (avatar != NULL)
+    g_string_free (avatar, TRUE);
+
   g_object_unref (conn);
 }
 
@@ -406,8 +565,21 @@ gabble_connection_request_avatar (TpSvcConnectionInterfaceAvatars *iface,
                                   DBusGMethodInvocation *context)
 {
   GabbleConnection *self = GABBLE_CONNECTION (iface);
+  TpBaseConnection *base = (TpBaseConnection *)self;
+  TpHandleRepoIface *contact_handles = tp_base_connection_get_handles (base,
+      TP_HANDLE_TYPE_CONTACT);
+  GError *err = NULL;
   LmMessageNode *vcard_node;
 
+  TP_BASE_CONNECTION_ERROR_IF_NOT_CONNECTED (base, context);
+
+  if (!tp_handle_is_valid (contact_handles, contact, &err))
+    {
+      dbus_g_method_return_error (context, err);
+      g_error_free (err);
+      return;
+    }
+
   if (gabble_vcard_manager_get_cached (self->vcard_manager,
       contact, &vcard_node))
     {
@@ -417,35 +589,106 @@ gabble_connection_request_avatar (TpSvcConnectionInterfaceAvatars *iface,
   else
     {
       gabble_vcard_manager_request (self->vcard_manager, contact, 0,
-          _request_avatar_cb, context, NULL, NULL);
+          _request_avatar_cb, context, NULL);
     }
 }
 
+static void
+emit_avatar_retrieved (TpSvcConnectionInterfaceAvatars *iface,
+                       TpHandle contact,
+                       LmMessageNode *vcard_node)
+{
+  const gchar *mime_type;
+  GString *avatar_str;
+  gchar *sha1;
+  GArray *arr;
+
+  if (!parse_avatar (vcard_node, &mime_type, &avatar_str, NULL))
+    return;
+
+  sha1 = sha1_hex (avatar_str->str, avatar_str->len);
+  arr = g_array_new (FALSE, FALSE, sizeof (gchar));
+  g_array_append_vals (arr, avatar_str->str, avatar_str->len);
+  tp_svc_connection_interface_avatars_emit_avatar_retrieved (iface, contact,
+      sha1, arr, mime_type);
+  g_array_free (arr, TRUE);
+  g_free (sha1);
+  g_string_free (avatar_str, TRUE);
+}
+
+static void
+request_avatars_cb (GabbleVCardManager *manager,
+                    GabbleVCardManagerRequest *request,
+                    TpHandle handle,
+                    LmMessageNode *vcard,
+                    GError *vcard_error,
+                    gpointer user_data)
+{
+  if (vcard_error != NULL)
+    return;
+
+  emit_avatar_retrieved (user_data, handle, vcard);
+}
+
+static void
+gabble_connection_request_avatars (TpSvcConnectionInterfaceAvatars *iface,
+                                   const GArray *contacts,
+                                   DBusGMethodInvocation *context)
+{
+  GabbleConnection *self = GABBLE_CONNECTION (iface);
+  TpBaseConnection *base = (TpBaseConnection *) self;
+  TpHandleRepoIface *contacts_repo =
+      tp_base_connection_get_handles (base, TP_HANDLE_TYPE_CONTACT);
+  GError *error = NULL;
+  guint i;
+
+  if (!tp_handles_are_valid (contacts_repo, contacts, FALSE, &error))
+    {
+      dbus_g_method_return_error (context, error);
+      g_error_free (error);
+      return;
+    }
+
+  for (i = 0; i < contacts->len; i++)
+    {
+      LmMessageNode *vcard_node;
+      TpHandle contact = g_array_index (contacts, TpHandle, i);
+
+      if (gabble_vcard_manager_get_cached (self->vcard_manager,
+            contact, &vcard_node))
+        {
+          emit_avatar_retrieved (iface, contact, vcard_node);
+        }
+      else
+        {
+          gabble_vcard_manager_request (self->vcard_manager,
+              contact, 0, request_avatars_cb, iface, NULL);
+        }
+    }
+
+  tp_svc_connection_interface_avatars_return_from_request_avatars (context);
+}
+
 
 struct _set_avatar_ctx {
   GabbleConnection *conn;
   DBusGMethodInvocation *invocation;
-  LmMessage *new_vcard_msg;
   GString *avatar;
-  gchar *mime_type;
 };
 
 
 static void
 _set_avatar_ctx_free (struct _set_avatar_ctx *ctx)
 {
-  lm_message_unref (ctx->new_vcard_msg);
   if (ctx->avatar)
       g_string_free (ctx->avatar, TRUE);
-  g_free (ctx->mime_type);
   g_free (ctx);
 }
 
 
 static void
 _set_avatar_cb2 (GabbleVCardManager *manager,
-                 GabbleVCardManagerRequest *request,
-                 TpHandle handle,
+                 GabbleVCardManagerEditRequest *request,
                  LmMessageNode *vcard,
                  GError *vcard_error,
                  gpointer user_data)
@@ -460,7 +703,7 @@ _set_avatar_cb2 (GabbleVCardManager *manager,
   else
     {
       GabblePresence *presence = ctx->conn->self_presence;
-      GError *error;
+      GError *error = NULL;
 
       g_free (presence->avatar_sha1);
       if (ctx->avatar)
@@ -491,67 +734,6 @@ _set_avatar_cb2 (GabbleVCardManager *manager,
 }
 
 
-static void
-_set_avatar_cb1 (GabbleVCardManager *manager,
-                 GabbleVCardManagerRequest *request,
-                 TpHandle handle,
-                 LmMessageNode *vcard,
-                 GError *vcard_error,
-                 gpointer user_data)
-{
-  struct _set_avatar_ctx *ctx = (struct _set_avatar_ctx *) user_data;
-  LmMessageNode *new_vcard, *photo_node, *type_node, *binval_node, *i, *next;
-  gchar *encoded;
-
-  if (NULL == vcard)
-    {
-      dbus_g_method_return_error (ctx->invocation, vcard_error);
-      _set_avatar_ctx_free (ctx);
-      return;
-    }
-
-  ctx->new_vcard_msg = lm_message_new (NULL, LM_MESSAGE_TYPE_UNKNOWN);
-  new_vcard = lm_message_node_add_child (ctx->new_vcard_msg->node, "vCard", "");
-  lm_message_node_set_attribute (new_vcard, "xmlns", NS_VCARD_TEMP);
-  lm_message_node_steal_children (new_vcard, vcard);
-
-  for (i = new_vcard->children; i; i = next)
-    {
-      next = i->next;
-      if (0 == strcmp (i->name, "PHOTO"))
-        {
-          lm_message_node_unlink (i);
-          lm_message_node_unref (i);
-        }
-    }
-
-  photo_node = lm_message_node_add_child (new_vcard, "PHOTO", "");
-
-  if (ctx->avatar)
-    {
-
-      type_node = lm_message_node_get_child (photo_node, "TYPE");
-
-      if (NULL == type_node)
-        type_node = lm_message_node_add_child (photo_node, "TYPE", "");
-
-      lm_message_node_set_value (type_node, ctx->mime_type);
-
-      binval_node = lm_message_node_get_child (photo_node, "BINVAL");
-
-      if (NULL == binval_node)
-        binval_node = lm_message_node_add_child (photo_node, "BINVAL", "");
-
-      encoded = base64_encode (ctx->avatar);
-      lm_message_node_set_value (binval_node, encoded);
-      g_free (encoded);
-    }
-
-  gabble_vcard_manager_replace (
-    manager, new_vcard, 0, _set_avatar_cb2, ctx, NULL, NULL);
-}
-
-
 /**
  * gabble_connection_set_avatar
  *
@@ -569,21 +751,29 @@ gabble_connection_set_avatar (TpSvcConnectionInterfaceAvatars *iface,
 {
   GabbleConnection *self = GABBLE_CONNECTION (iface);
   TpBaseConnection *base = (TpBaseConnection *)self;
-  struct _set_avatar_ctx *ctx = g_new0 (struct _set_avatar_ctx, 1);
+  struct _set_avatar_ctx *ctx;
+  gchar *value = NULL;
+  gchar *base64;
 
+  TP_BASE_CONNECTION_ERROR_IF_NOT_CONNECTED (base, context);
+
+  ctx = g_new0 (struct _set_avatar_ctx, 1);
   ctx->conn = self;
   ctx->invocation = context;
   if (avatar)
     {
       ctx->avatar = g_string_new_len (avatar->data, avatar->len);
-      ctx->mime_type = g_strdup (mime_type);
+      base64 = base64_encode (avatar->len, avatar->data);
+      value = g_strdup_printf ("%s %s", mime_type, base64);
+      g_free (base64);
     }
 
   DEBUG ("called");
-  gabble_vcard_manager_invalidate_cache (self->vcard_manager,
-      base->self_handle);
-  gabble_vcard_manager_request (self->vcard_manager,
-      base->self_handle, 0, _set_avatar_cb1, ctx, NULL, NULL);
+
+  gabble_vcard_manager_edit (self->vcard_manager, 0,
+      _set_avatar_cb2, ctx, (GObject *) self,
+      1, "PHOTO", value);
+  g_free (value);
 }
 
 
@@ -617,13 +807,16 @@ conn_avatars_init (GabbleConnection *conn)
 void
 conn_avatars_iface_init (gpointer g_iface, gpointer iface_data)
 {
-  TpSvcConnectionInterfaceAvatarsClass *klass = (TpSvcConnectionInterfaceAvatarsClass *) g_iface;
+  TpSvcConnectionInterfaceAvatarsClass *klass =
+    (TpSvcConnectionInterfaceAvatarsClass *) g_iface;
 
 #define IMPLEMENT(x) tp_svc_connection_interface_avatars_implement_##x (\
     klass, gabble_connection_##x)
   IMPLEMENT(get_avatar_requirements);
   IMPLEMENT(get_avatar_tokens);
+  IMPLEMENT(get_known_avatar_tokens);
   IMPLEMENT(request_avatar);
+  IMPLEMENT(request_avatars);
   IMPLEMENT(set_avatar);
   IMPLEMENT(clear_avatar);
 #undef IMPLEMENT
