@@ -1,15 +1,54 @@
 #include "config.h"
 
-#ifdef ENABLE_DEBUG
-
 #include <stdarg.h>
+#include <unistd.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <fcntl.h>
+#include <errno.h>
 
 #include <glib.h>
+#include <glib/gstdio.h>
 
 #include <telepathy-glib/debug.h>
 
 #include "debug.h"
 
+void
+gabble_debug_set_log_file_from_env (void)
+{
+  const gchar *output_file;
+  int out;
+
+  output_file = g_getenv ("GABBLE_LOGFILE");
+  if (output_file == NULL)
+    return;
+
+  out = g_open (output_file, O_WRONLY | O_CREAT, 0644);
+  if (out == -1)
+    {
+      g_warning ("Can't open logfile '%s': %s", output_file,
+          g_strerror (errno));
+      return;
+    }
+
+  if (dup2 (out, STDOUT_FILENO) == -1)
+    {
+      g_warning ("Error when duplicating stdout file descriptor: %s",
+          g_strerror (errno));
+      return;
+    }
+
+  if (dup2 (out, STDERR_FILENO) == -1)
+    {
+      g_warning ("Error when duplicating stderr file descriptor: %s",
+          g_strerror (errno));
+      return;
+    }
+}
+
+#ifdef ENABLE_DEBUG
+
 static GabbleDebugFlags flags = 0;
 
 static GDebugKey keys[] = {
@@ -20,10 +59,13 @@ static GDebugKey keys[] = {
   { "properties",    GABBLE_DEBUG_PROPERTIES },
   { "roomlist",      GABBLE_DEBUG_ROOMLIST },
   { "media-channel", GABBLE_DEBUG_MEDIA },
+  { "im",            GABBLE_DEBUG_IM },
   { "muc",           GABBLE_DEBUG_MUC },
   { "connection",    GABBLE_DEBUG_CONNECTION },
   { "persist",       GABBLE_DEBUG_PERSIST },
   { "vcard",         GABBLE_DEBUG_VCARD },
+  { "jid",           GABBLE_DEBUG_JID },
+  { "olpc",          GABBLE_DEBUG_OLPC },
   { 0, },
 };
 
@@ -39,7 +81,8 @@ void gabble_debug_set_flags_from_env ()
   if (flags_string)
     {
       tp_debug_set_flags_from_env ("GABBLE_DEBUG");
-      gabble_debug_set_flags (g_parse_debug_string (flags_string, keys, nkeys));
+      gabble_debug_set_flags (g_parse_debug_string (flags_string, keys,
+            nkeys));
     }
 }
 
@@ -67,4 +110,3 @@ void gabble_debug (GabbleDebugFlags flag,
 }
 
 #endif /* ENABLE_DEBUG */
-
