@@ -4,6 +4,12 @@
 
 #include "config.h"
 
+G_BEGIN_DECLS
+
+void gabble_debug_set_log_file_from_env (void);
+
+G_END_DECLS
+
 #ifdef ENABLE_DEBUG
 
 #include <glib.h>
@@ -24,6 +30,9 @@ typedef enum
   GABBLE_DEBUG_IM            = 1 << 9,
   GABBLE_DEBUG_PERSIST       = 1 << 10,
   GABBLE_DEBUG_VCARD         = 1 << 11,
+  GABBLE_DEBUG_PIPELINE      = 1 << 12,
+  GABBLE_DEBUG_JID           = 1 << 13,
+  GABBLE_DEBUG_OLPC          = 1 << 14,
 } GabbleDebugFlags;
 
 void gabble_debug_set_flags_from_env ();
@@ -37,9 +46,9 @@ G_END_DECLS
 #ifdef DEBUG_FLAG
 
 #define DEBUG(format, ...) \
-  gabble_debug(DEBUG_FLAG, "%s: " format, G_STRFUNC, ##__VA_ARGS__)
+  gabble_debug (DEBUG_FLAG, "%s: " format, G_STRFUNC, ##__VA_ARGS__)
 
-#define DEBUGGING gabble_debug_flag_is_set(DEBUG_FLAG)
+#define DEBUGGING gabble_debug_flag_is_set (DEBUG_FLAG)
 
 #define NODE_DEBUG(n, s) \
 G_STMT_START { \
@@ -54,7 +63,7 @@ G_STMT_START { \
 
 #ifdef DEBUG_FLAG
 
-#define DEBUG(format, ...)
+#define DEBUG(format, ...) do { } while (0);
 
 #define DEBUGGING 0
 
