@@ -21,6 +21,8 @@
  * -- LET'S DISCO!!!  \o/ \o_ _o/ /\o/\ _/o/- -\o\_ --
  */
 
+#include "disco.h"
+
 #define DBUS_API_SUBJECT_TO_CHANGE
 
 #include <dbus/dbus-glib.h>
@@ -31,7 +33,6 @@
 #define DEBUG_FLAG GABBLE_DEBUG_DISCO
 
 #include "debug.h"
-#include "disco.h"
 #include "gabble-connection.h"
 #include "gabble-error.h"
 #include "namespaces.h"
@@ -276,7 +277,8 @@ delete_request (GabbleDiscoRequest *request)
 
   if (NULL != request->bound_object)
     {
-      g_object_weak_unref (request->bound_object, notify_delete_request, request);
+      g_object_weak_unref (request->bound_object, notify_delete_request,
+          request);
     }
 
   if (0 != request->timer_id)
@@ -293,7 +295,7 @@ static gboolean
 timeout_request (gpointer data)
 {
   GabbleDiscoRequest *request = (GabbleDiscoRequest*) data;
-  GError *err;
+  GError *err /* doesn't need initializing */;
   g_return_val_if_fail (data != NULL, FALSE);
 
   err = g_error_new (GABBLE_DISCO_ERROR, GABBLE_DISCO_ERROR_TIMEOUT,
@@ -312,7 +314,7 @@ timeout_request (gpointer data)
 static void
 cancel_request (GabbleDiscoRequest *request)
 {
-  GError *err;
+  GError *err /* doesn't need initializing */;
 
   g_assert (request != NULL);
 
@@ -327,6 +329,21 @@ cancel_request (GabbleDiscoRequest *request)
   delete_request (request);
 }
 
+static const char *
+disco_type_to_xmlns (GabbleDiscoType type)
+{
+  switch (type) {
+    case GABBLE_DISCO_TYPE_INFO:
+      return NS_DISCO_INFO;
+    case GABBLE_DISCO_TYPE_ITEMS:
+      return NS_DISCO_ITEMS;
+    default:
+      g_assert_not_reached ();
+  }
+
+  return NULL;
+}
+
 static LmHandlerResult
 request_reply_cb (GabbleConnection *conn, LmMessage *sent_msg,
                   LmMessage *reply_msg, GObject *object, gpointer user_data)
@@ -342,7 +359,8 @@ request_reply_cb (GabbleConnection *conn, LmMessage *sent_msg,
   if (!g_list_find (priv->requests, request))
     return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
 
-  query_node = lm_message_node_get_child (reply_msg->node, "query");
+  query_node = lm_message_node_get_child_with_namespace (reply_msg->node,
+      "query", disco_type_to_xmlns (request->type));
 
   if (lm_message_get_sub_type (reply_msg) == LM_MESSAGE_SUB_TYPE_ERROR)
     {
@@ -418,7 +436,8 @@ gabble_disco_request (GabbleDisco *self, GabbleDiscoType type,
  * @type: type of request
  * @jid: Jabber ID to request on
  * @node: node to request on @jid, or NULL
- * @timeout: the time until the request fails, in milliseconds (1/1000ths of a second)
+ * @timeout: the time until the request fails, in milliseconds (1/1000ths of
+ *           a second)
  * @callback: #GabbleDiscoCb to call on request fullfilment
  * @object: GObject to bind request to. the callback will not be
  *          called if this object has been unrefed. NULL if not needed
@@ -439,7 +458,6 @@ gabble_disco_request_with_timeout (GabbleDisco *self, GabbleDiscoType type,
   GabbleDiscoRequest *request;
   LmMessage *msg;
   LmMessageNode *lm_node;
-  const gchar *xmlns;
 
   request = g_slice_new0 (GabbleDiscoRequest);
   request->disco = self;
@@ -462,19 +480,7 @@ gabble_disco_request_with_timeout (GabbleDisco *self, GabbleDiscoType type,
                                            LM_MESSAGE_SUB_TYPE_GET);
   lm_node = lm_message_node_add_child (msg->node, "query", NULL);
 
-  switch (type) {
-    case GABBLE_DISCO_TYPE_INFO:
-      xmlns = NS_DISCO_INFO;
-      break;
-    case GABBLE_DISCO_TYPE_ITEMS:
-      xmlns = NS_DISCO_ITEMS;
-      break;
-    default:
-      g_assert_not_reached ();
-      return NULL;
-  }
-
-  lm_message_node_set_attribute (lm_node, "xmlns", xmlns);
+  lm_message_node_set_attribute (lm_node, "xmlns", disco_type_to_xmlns (type));
 
   if (node)
     {
@@ -696,9 +702,7 @@ disco_items_cb (GabbleDisco *disco,
       goto out;
     }
 
-  iter = result->children;
-
-  for (; iter; iter = iter->next)
+  for (iter = result->children; iter; iter = iter->next)
     {
       if (0 != strcmp (iter->name, "item"))
         continue;
@@ -706,7 +710,8 @@ disco_items_cb (GabbleDisco *disco,
       item_jid = lm_message_node_get_attribute (iter, "jid");
 
       if (NULL != item_jid &&
-          !g_hash_table_lookup_extended (pipeline->remaining_items, item_jid, &key, &value))
+          !g_hash_table_lookup_extended (pipeline->remaining_items, item_jid,
+            &key, &value))
         {
           gchar *tmp = g_strdup (item_jid);
           DEBUG ("discovered service item: %s", tmp);
@@ -831,8 +836,10 @@ services_cb (gpointer pipeline, GabbleDiscoItem *item, gpointer user_data)
   my_item->type = g_strdup (item->type);
   my_item->category = g_strdup (item->category);
 
-  my_item->features = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
-  g_hash_table_foreach  (item->features, service_feature_copy_one, my_item->features);
+  my_item->features = g_hash_table_new_full (g_str_hash, g_str_equal, g_free,
+      NULL);
+  g_hash_table_foreach  (item->features, service_feature_copy_one,
+      my_item->features);
 
   priv->service_cache = g_slist_prepend (priv->service_cache, my_item);
 }
