@@ -1,7 +1,7 @@
 /*
  * gabble-connection-manager.c - Source for GabbleConnectionManager
- * Copyright (C) 2005 Collabora Ltd.
- * Copyright (C) 2005 Nokia Corporation
+ * Copyright (C) 2005-2007 Collabora Ltd.
+ * Copyright (C) 2005-2007 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,6 +18,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "gabble-connection-manager.h"
+
 #include <dbus/dbus-protocol.h>
 #include <dbus/dbus-glib.h>
 #include <stdio.h>
@@ -28,8 +30,6 @@
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/errors.h>
 
-#include "gabble-connection-manager.h"
-
 G_DEFINE_TYPE(GabbleConnectionManager,
     gabble_connection_manager,
     TP_TYPE_BASE_CONNECTION_MANAGER)
@@ -48,7 +48,7 @@ static TpBaseConnection *_gabble_connection_manager_new_connection (
 static void
 gabble_connection_manager_class_init (GabbleConnectionManagerClass *klass)
 {
-  TpBaseConnectionManagerClass *base_class = 
+  TpBaseConnectionManagerClass *base_class =
     (TpBaseConnectionManagerClass *)klass;
 
   base_class->new_connection = _gabble_connection_manager_new_connection;
@@ -61,8 +61,6 @@ gabble_connection_manager_class_init (GabbleConnectionManagerClass *klass)
 typedef struct _GabbleParams GabbleParams;
 
 struct _GabbleParams {
-  guint set_mask;
-
   gchar *account;
   gchar *password;
   gchar *server;
@@ -106,24 +104,87 @@ enum {
 };
 
 static const TpCMParamSpec jabber_params[] = {
-  { "account", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, TP_CONN_MGR_PARAM_FLAG_REQUIRED | TP_CONN_MGR_PARAM_FLAG_REGISTER, NULL, G_STRUCT_OFFSET(GabbleParams, account) },
-  { "password", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, TP_CONN_MGR_PARAM_FLAG_REQUIRED | TP_CONN_MGR_PARAM_FLAG_REGISTER, NULL, G_STRUCT_OFFSET(GabbleParams, password) },
-  { "server", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL, G_STRUCT_OFFSET(GabbleParams, server) },
-  { "resource", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GABBLE_PARAMS_DEFAULT_RESOURCE, G_STRUCT_OFFSET(GabbleParams, resource) },
-  { "priority", DBUS_TYPE_INT16_AS_STRING, G_TYPE_INT, TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(0), G_STRUCT_OFFSET(GabbleParams, priority) },
-  { "port", DBUS_TYPE_UINT16_AS_STRING, G_TYPE_UINT, TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(GABBLE_PARAMS_DEFAULT_PORT), G_STRUCT_OFFSET(GabbleParams, port) },
-  { "old-ssl", DBUS_TYPE_BOOLEAN_AS_STRING, G_TYPE_BOOLEAN, TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(FALSE), G_STRUCT_OFFSET(GabbleParams, old_ssl) },
-  { "register", DBUS_TYPE_BOOLEAN_AS_STRING, G_TYPE_BOOLEAN, TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(FALSE), G_STRUCT_OFFSET(GabbleParams, do_register) },
-  { "low-bandwidth", DBUS_TYPE_BOOLEAN_AS_STRING, G_TYPE_BOOLEAN, TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(FALSE), G_STRUCT_OFFSET(GabbleParams, low_bandwidth) },
-  { "https-proxy-server", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL, G_STRUCT_OFFSET(GabbleParams, https_proxy_server) },
-  { "https-proxy-port", DBUS_TYPE_UINT16_AS_STRING, G_TYPE_UINT, TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(GABBLE_PARAMS_DEFAULT_HTTPS_PROXY_PORT), G_STRUCT_OFFSET(GabbleParams, https_proxy_port) },
-  { "fallback-conference-server", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL, G_STRUCT_OFFSET(GabbleParams, fallback_conference_server) },
-  { "stun-server", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL, G_STRUCT_OFFSET(GabbleParams, stun_server) },
-  { "stun-port", DBUS_TYPE_UINT16_AS_STRING, G_TYPE_UINT, TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(GABBLE_PARAMS_DEFAULT_STUN_PORT), G_STRUCT_OFFSET(GabbleParams, stun_port) },
-  { "ignore-ssl-errors", DBUS_TYPE_BOOLEAN_AS_STRING, G_TYPE_BOOLEAN, TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(FALSE), G_STRUCT_OFFSET(GabbleParams, ignore_ssl_errors) },
-  { "alias", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL, G_STRUCT_OFFSET(GabbleParams, alias) },
-  { "mac", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL, G_STRUCT_OFFSET(GabbleParams, auth_mac) },
-  { "btid", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL, G_STRUCT_OFFSET(GabbleParams, auth_btid) },
+  { "account", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING,
+    TP_CONN_MGR_PARAM_FLAG_REQUIRED | TP_CONN_MGR_PARAM_FLAG_REGISTER, NULL,
+    G_STRUCT_OFFSET(GabbleParams, account),
+    /* FIXME: validate the JID according to the RFC */
+    tp_cm_param_filter_string_nonempty, NULL },
+  { "password", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING,
+    TP_CONN_MGR_PARAM_FLAG_REQUIRED | TP_CONN_MGR_PARAM_FLAG_REGISTER, NULL,
+    G_STRUCT_OFFSET(GabbleParams, password), NULL, NULL },
+
+  { "server", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL,
+    G_STRUCT_OFFSET(GabbleParams, server),
+    /* FIXME: validate the server properly */
+    tp_cm_param_filter_string_nonempty, NULL },
+
+  { "resource", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING,
+    TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GABBLE_PARAMS_DEFAULT_RESOURCE,
+    G_STRUCT_OFFSET(GabbleParams, resource),
+    /* FIXME: validate the resource according to the RFC */
+    tp_cm_param_filter_string_nonempty, NULL },
+
+  { "priority", DBUS_TYPE_INT16_AS_STRING, G_TYPE_INT,
+    TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(0),
+    G_STRUCT_OFFSET(GabbleParams, priority), NULL, NULL },
+
+  { "port", DBUS_TYPE_UINT16_AS_STRING, G_TYPE_UINT,
+    TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT,
+    GINT_TO_POINTER(GABBLE_PARAMS_DEFAULT_PORT),
+    G_STRUCT_OFFSET(GabbleParams, port),
+    tp_cm_param_filter_uint_nonzero, NULL },
+
+  { "old-ssl", DBUS_TYPE_BOOLEAN_AS_STRING, G_TYPE_BOOLEAN,
+    TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(FALSE),
+    G_STRUCT_OFFSET(GabbleParams, old_ssl), NULL, NULL },
+
+  { "register", DBUS_TYPE_BOOLEAN_AS_STRING, G_TYPE_BOOLEAN,
+    TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(FALSE),
+    G_STRUCT_OFFSET(GabbleParams, do_register), NULL, NULL },
+
+  { "low-bandwidth", DBUS_TYPE_BOOLEAN_AS_STRING, G_TYPE_BOOLEAN,
+    TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(FALSE),
+    G_STRUCT_OFFSET(GabbleParams, low_bandwidth), NULL, NULL },
+
+  { "https-proxy-server", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL,
+    G_STRUCT_OFFSET(GabbleParams, https_proxy_server),
+    /* FIXME: validate properly */
+    tp_cm_param_filter_string_nonempty, NULL },
+  { "https-proxy-port", DBUS_TYPE_UINT16_AS_STRING, G_TYPE_UINT,
+    TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT,
+    GINT_TO_POINTER(GABBLE_PARAMS_DEFAULT_HTTPS_PROXY_PORT),
+    G_STRUCT_OFFSET(GabbleParams, https_proxy_port),
+    tp_cm_param_filter_uint_nonzero, NULL },
+
+  { "fallback-conference-server", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING,
+    0, NULL, G_STRUCT_OFFSET(GabbleParams, fallback_conference_server),
+    /* FIXME: validate properly */
+    tp_cm_param_filter_string_nonempty, NULL },
+
+  { "stun-server", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL,
+    G_STRUCT_OFFSET(GabbleParams, stun_server),
+    /* FIXME: validate properly */
+    tp_cm_param_filter_string_nonempty, NULL },
+  { "stun-port", DBUS_TYPE_UINT16_AS_STRING, G_TYPE_UINT,
+    TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT,
+    GINT_TO_POINTER(GABBLE_PARAMS_DEFAULT_STUN_PORT),
+    G_STRUCT_OFFSET(GabbleParams, stun_port),
+    tp_cm_param_filter_uint_nonzero, NULL },
+
+  { "ignore-ssl-errors", DBUS_TYPE_BOOLEAN_AS_STRING, G_TYPE_BOOLEAN,
+    TP_CONN_MGR_PARAM_FLAG_HAS_DEFAULT, GINT_TO_POINTER(FALSE),
+    G_STRUCT_OFFSET(GabbleParams, ignore_ssl_errors), NULL, NULL },
+
+  { "alias", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL,
+    G_STRUCT_OFFSET(GabbleParams, alias),
+    /* setting a 0-length alias makes no sense */
+    tp_cm_param_filter_string_nonempty, NULL },
+
+  { "mac", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL,
+    G_STRUCT_OFFSET(GabbleParams, auth_mac), NULL, NULL },
+  { "btid", DBUS_TYPE_STRING_AS_STRING, G_TYPE_STRING, 0, NULL,
+    G_STRUCT_OFFSET(GabbleParams, auth_btid), NULL, NULL },
+
   { NULL, NULL, 0, 0, NULL, 0 }
 };
 
@@ -152,13 +213,10 @@ free_params (void *p)
   g_slice_free (GabbleParams, params);
 }
 
-static const TpCMProtocolSpec _gabble_protocols[] = {
+const TpCMProtocolSpec gabble_protocols[] = {
   { "jabber", jabber_params, alloc_params, free_params },
   { NULL, NULL }
 };
-const TpCMProtocolSpec *gabble_protocols = _gabble_protocols;
-
-/* dbus-exported methods */
 
 #define SET_PROPERTY_IF_PARAM_SET(prop, param, member) \
   if (tp_intset_is_member (params_present, param)) \
@@ -166,7 +224,6 @@ const TpCMProtocolSpec *gabble_protocols = _gabble_protocols;
       g_object_set (conn, prop, member, NULL); \
     }
 
-
 static TpBaseConnection *
 _gabble_connection_manager_new_connection (TpBaseConnectionManager *self,
                                            const gchar *proto,
@@ -189,7 +246,7 @@ _gabble_connection_manager_new_connection (TpBaseConnectionManager *self,
   SET_PROPERTY_IF_PARAM_SET ("resource", JABBER_PARAM_RESOURCE,
                              params->resource);
   SET_PROPERTY_IF_PARAM_SET ("priority", JABBER_PARAM_PRIORITY,
-                             CLAMP (params->priority, G_MININT8, G_MAXINT8));
+                             (gint8) CLAMP (params->priority, G_MININT8, G_MAXINT8));
   SET_PROPERTY_IF_PARAM_SET ("port", JABBER_PARAM_PORT, params->port);
   SET_PROPERTY_IF_PARAM_SET ("old-ssl", JABBER_PARAM_OLD_SSL, params->old_ssl);
   SET_PROPERTY_IF_PARAM_SET ("register", JABBER_PARAM_REGISTER,
