@@ -18,6 +18,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "gabble-im-channel.h"
+
 #include <dbus/dbus-glib.h>
 #include <loudmouth/loudmouth.h>
 #include <stdio.h>
@@ -31,7 +33,6 @@
 #include "gabble-connection.h"
 #include "presence.h"
 #include "presence-cache.h"
-#include "handles.h"
 #include "roster.h"
 #include <telepathy-glib/dbus.h>
 #include <telepathy-glib/enums.h>
@@ -39,9 +40,6 @@
 #include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/channel-iface.h>
 #include <telepathy-glib/svc-channel.h>
-#include "text-mixin.h"
-
-#include "gabble-im-channel.h"
 
 static void channel_iface_init (gpointer, gpointer);
 static void text_iface_init (gpointer, gpointer);
@@ -51,7 +49,8 @@ G_DEFINE_TYPE_WITH_CODE (GabbleIMChannel, gabble_im_channel, G_TYPE_OBJECT,
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL, channel_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_TYPE_TEXT, text_iface_init);
     G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_IFACE, NULL);
-    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_INTERFACE_CHAT_STATE, chat_state_iface_init));
+    G_IMPLEMENT_INTERFACE (TP_TYPE_SVC_CHANNEL_INTERFACE_CHAT_STATE,
+      chat_state_iface_init));
 
 
 /* properties */
@@ -100,19 +99,20 @@ gabble_im_channel_constructor (GType type, guint n_props,
   GabbleIMChannelPrivate *priv;
   TpBaseConnection *conn;
   DBusGConnection *bus;
-  gboolean valid, send_nick;
+  gboolean send_nick;
+  TpHandleRepoIface *contact_handles;
 
   obj = G_OBJECT_CLASS (gabble_im_channel_parent_class)->
            constructor (type, n_props, props);
   priv = GABBLE_IM_CHANNEL_GET_PRIVATE (GABBLE_IM_CHANNEL (obj));
   conn = (TpBaseConnection *)priv->conn;
+  contact_handles = tp_base_connection_get_handles (conn,
+      TP_HANDLE_TYPE_CONTACT);
 
-  valid = tp_handle_ref (conn->handles[TP_HANDLE_TYPE_CONTACT],
-      priv->handle);
-  g_assert (valid);
+  tp_handle_ref (contact_handles, priv->handle);
 
-  priv->peer_jid = g_strdup (tp_handle_inspect (
-        conn->handles[TP_HANDLE_TYPE_CONTACT], priv->handle));
+  priv->peer_jid = g_strdup (tp_handle_inspect (contact_handles,
+        priv->handle));
 
   bus = tp_get_bus ();
   dbus_g_connection_register_g_object (bus, priv->object_path, obj);
@@ -123,9 +123,8 @@ gabble_im_channel_constructor (GType type, guint n_props,
   else
     send_nick = TRUE;
 
-  tp_text_mixin_init (obj, G_STRUCT_OFFSET (GabbleIMChannel, text),
-      conn->handles[TP_HANDLE_TYPE_CONTACT],
-      send_nick);
+  gabble_text_mixin_init (obj, G_STRUCT_OFFSET (GabbleIMChannel, text),
+      contact_handles, send_nick);
 
   tp_text_mixin_set_message_types (obj,
       TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL,
@@ -182,6 +181,9 @@ gabble_im_channel_set_property (GObject     *object,
       priv->object_path = g_value_dup_string (value);
       break;
     case PROP_HANDLE:
+      /* we don't ref it here because we don't necessarily have access to the
+       * contact repo yet - instead we ref it in the constructor.
+       */
       priv->handle = g_value_get_uint (value);
       break;
     case PROP_HANDLE_TYPE:
@@ -206,7 +208,8 @@ gabble_im_channel_class_init (GabbleIMChannelClass *gabble_im_channel_class)
   GObjectClass *object_class = G_OBJECT_CLASS (gabble_im_channel_class);
   GParamSpec *param_spec;
 
-  g_type_class_add_private (gabble_im_channel_class, sizeof (GabbleIMChannelPrivate));
+  g_type_class_add_private (gabble_im_channel_class,
+      sizeof (GabbleIMChannelPrivate));
 
   object_class->constructor = gabble_im_channel_constructor;
 
@@ -216,9 +219,12 @@ gabble_im_channel_class_init (GabbleIMChannelClass *gabble_im_channel_class)
   object_class->dispose = gabble_im_channel_dispose;
   object_class->finalize = gabble_im_channel_finalize;
 
-  g_object_class_override_property (object_class, PROP_OBJECT_PATH, "object-path");
-  g_object_class_override_property (object_class, PROP_CHANNEL_TYPE, "channel-type");
-  g_object_class_override_property (object_class, PROP_HANDLE_TYPE, "handle-type");
+  g_object_class_override_property (object_class, PROP_OBJECT_PATH,
+      "object-path");
+  g_object_class_override_property (object_class, PROP_CHANNEL_TYPE,
+      "channel-type");
+  g_object_class_override_property (object_class, PROP_HANDLE_TYPE,
+      "handle-type");
   g_object_class_override_property (object_class, PROP_HANDLE, "handle");
 
   param_spec = g_param_spec_object ("connection", "GabbleConnection object",
@@ -234,7 +240,7 @@ gabble_im_channel_class_init (GabbleIMChannelClass *gabble_im_channel_class)
   tp_text_mixin_class_init (object_class, G_STRUCT_OFFSET (GabbleIMChannelClass, text_class));
 }
 
-void
+static void
 gabble_im_channel_dispose (GObject *object)
 {
   GabbleIMChannel *self = GABBLE_IM_CHANNEL (object);
@@ -273,11 +279,12 @@ gabble_im_channel_dispose (GObject *object)
       {
         if (cap_chat_states)
           {
-          /* Set the chat state of the channel on gone 
+          /* Set the chat state of the channel on gone
            * (Channel.Interface.ChatState) */
           gabble_text_mixin_send (G_OBJECT (self),
-              TP_CHANNEL_TEXT_MESSAGE_TYPE_NOTICE, 0, TP_CHANNEL_CHAT_STATE_GONE,
-              priv->peer_jid, NULL, priv->conn, FALSE /* emit_signal */, NULL);
+              TP_CHANNEL_TEXT_MESSAGE_TYPE_NOTICE, 0,
+              TP_CHANNEL_CHAT_STATE_GONE, priv->peer_jid, NULL, priv->conn,
+              FALSE /* emit_signal */, NULL);
           }
 
         tp_svc_channel_emit_closed (self);
@@ -287,17 +294,20 @@ gabble_im_channel_dispose (GObject *object)
     G_OBJECT_CLASS (gabble_im_channel_parent_class)->dispose (object);
 }
 
-void
+static void
 gabble_im_channel_finalize (GObject *object)
 {
   GabbleIMChannel *self = GABBLE_IM_CHANNEL (object);
   GabbleIMChannelPrivate *priv = GABBLE_IM_CHANNEL_GET_PRIVATE (self);
   TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
+  TpHandleRepoIface *contact_handles = tp_base_connection_get_handles (conn,
+      TP_HANDLE_TYPE_CONTACT);
 
   /* free any data held directly by the object here */
 
-  tp_handle_unref (conn->handles[TP_HANDLE_TYPE_CONTACT],
-      priv->handle);
+  DEBUG ("%p", object);
+
+  tp_handle_unref (contact_handles, priv->handle);
 
   g_free (priv->object_path);
   g_free (priv->peer_jid);
@@ -311,12 +321,13 @@ gabble_im_channel_finalize (GObject *object)
  * _gabble_im_channel_receive
  *
  */
-gboolean _gabble_im_channel_receive (GabbleIMChannel *chan,
-                                     TpChannelTextMessageType type,
-                                     TpHandle sender,
-                                     const char *from,
-                                     time_t timestamp,
-                                     const char *text)
+void
+_gabble_im_channel_receive (GabbleIMChannel *chan,
+                            TpChannelTextMessageType type,
+                            TpHandle sender,
+                            const char *from,
+                            time_t timestamp,
+                            const char *text)
 {
   GabbleIMChannelPrivate *priv;
 
@@ -333,12 +344,12 @@ gboolean _gabble_im_channel_receive (GabbleIMChannel *chan,
   if (timestamp == 0)
       timestamp = time (NULL);
 
-  return tp_text_mixin_receive (G_OBJECT (chan), type, sender, timestamp, text);
+  tp_text_mixin_receive (G_OBJECT (chan), type, sender, timestamp, text);
 }
 
 /**
  * _gabble_im_channel_state_receive
- *			      
+ *
  * Send the D-BUS signal ChatStateChanged
  * on org.freedesktop.Telepathy.Channel.Interface.ChatState
  */
@@ -349,7 +360,7 @@ _gabble_im_channel_state_receive (GabbleIMChannel *chan,
 {
   GabbleIMChannelPrivate *priv;
 
-  g_assert (state <= LAST_TP_CHANNEL_CHAT_STATE);
+  g_assert (state < NUM_TP_CHANNEL_CHAT_STATES);
   g_assert (GABBLE_IS_IM_CHANNEL (chan));
   priv = GABBLE_IM_CHANNEL_GET_PRIVATE (chan);
 
@@ -383,6 +394,8 @@ gabble_im_channel_close (TpSvcChannel *iface,
 
   if (!priv->closed)
     {
+      tp_svc_channel_emit_closed (self);
+
       if (presence && (presence->caps & PRESENCE_CAP_CHAT_STATES))
         {
           /* Set the chat state of the channel on gone
@@ -392,11 +405,10 @@ gabble_im_channel_close (TpSvcChannel *iface,
               TP_CHANNEL_CHAT_STATE_GONE, priv->peer_jid, NULL, priv->conn,
               FALSE /* emit_signal */, NULL);
         }
+
       priv->closed = TRUE;
     }
 
-  tp_svc_channel_emit_closed (self);
-
   tp_svc_channel_return_from_close (context);
 }
 
@@ -448,7 +460,7 @@ gabble_im_channel_get_interfaces (TpSvcChannel *iface,
                                   DBusGMethodInvocation *context)
 {
   const char *interfaces[] = {
-      TP_IFACE_CHANNEL_INTERFACE_CHAT_STATE, 
+      TP_IFACE_CHANNEL_INTERFACE_CHAT_STATE,
       NULL };
 
   tp_svc_channel_return_from_get_interfaces (context, interfaces);
@@ -521,7 +533,7 @@ gabble_im_channel_set_chat_state (TpSvcChannelInterfaceChatState *iface,
 
   if (presence && (presence->caps & PRESENCE_CAP_CHAT_STATES))
     {
-      if (state > LAST_TP_CHANNEL_CHAT_STATE)
+      if (state >= NUM_TP_CHANNEL_CHAT_STATES)
         {
           DEBUG ("invalid state %u", state);
 
@@ -557,7 +569,7 @@ gabble_im_channel_set_chat_state (TpSvcChannelInterfaceChatState *iface,
 }
 
 static void
-channel_iface_init(gpointer g_iface, gpointer iface_data)
+channel_iface_init (gpointer g_iface, gpointer iface_data)
 {
   TpSvcChannelClass *klass = (TpSvcChannelClass *)g_iface;
 
@@ -571,7 +583,7 @@ channel_iface_init(gpointer g_iface, gpointer iface_data)
 }
 
 static void
-text_iface_init(gpointer g_iface, gpointer iface_data)
+text_iface_init (gpointer g_iface, gpointer iface_data)
 {
   TpSvcChannelTypeTextClass *klass = (TpSvcChannelTypeTextClass *)g_iface;
 
@@ -583,9 +595,10 @@ text_iface_init(gpointer g_iface, gpointer iface_data)
 }
 
 static void
-chat_state_iface_init(gpointer g_iface, gpointer iface_data)
+chat_state_iface_init (gpointer g_iface, gpointer iface_data)
 {
-  TpSvcChannelInterfaceChatStateClass *klass = (TpSvcChannelInterfaceChatStateClass *)g_iface;
+  TpSvcChannelInterfaceChatStateClass *klass =
+    (TpSvcChannelInterfaceChatStateClass *)g_iface;
 #define IMPLEMENT(x) tp_svc_channel_interface_chat_state_implement_##x (\
     klass, gabble_im_channel_##x)
   IMPLEMENT(set_chat_state);
