@@ -19,6 +19,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "gabble-media-session.h"
+
 #include <dbus/dbus-glib.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -29,7 +31,6 @@
 
 #include <telepathy-glib/debug-ansi.h>
 #include "debug.h"
-#include "handles.h"
 #include "namespaces.h"
 #include "util.h"
 
@@ -43,7 +44,6 @@
 #include "presence-cache.h"
 #include "presence.h"
 
-#include "gabble-media-session.h"
 #include "gabble-signals-marshal.h"
 
 static void session_handler_iface_init (gpointer, gpointer);
@@ -55,7 +55,7 @@ G_DEFINE_TYPE_WITH_CODE(GabbleMediaSession,
       session_handler_iface_init)
     )
 
-#define DEFAULT_SESSION_TIMEOUT 50000
+#define DEFAULT_SESSION_TIMEOUT 60000
 
 #define GTALK_STREAM_NAME "gtalk"
 
@@ -124,12 +124,18 @@ typedef struct {
 
 static const SessionStateDescription session_states[] =
 {
-    { "JS_STATE_PENDING_CREATED",       TP_ANSI_BOLD_ON TP_ANSI_FG_BLACK TP_ANSI_BG_WHITE   },
-    { "JS_STATE_PENDING_INITIATE_SENT", TP_ANSI_BOLD_ON               TP_ANSI_BG_CYAN    },
-    { "JS_STATE_PENDING_INITIATED",     TP_ANSI_BOLD_ON               TP_ANSI_BG_MAGENTA },
-    { "JS_STATE_PENDING_ACCEPT_SENT",   TP_ANSI_BOLD_ON               TP_ANSI_BG_CYAN    },
-    { "JS_STATE_ACTIVE",                TP_ANSI_BOLD_ON               TP_ANSI_BG_BLUE    },
-    { "JS_STATE_ENDED",                                            TP_ANSI_BG_RED     }
+    { "JS_STATE_PENDING_CREATED",
+      TP_ANSI_BOLD_ON TP_ANSI_FG_BLACK TP_ANSI_BG_WHITE   },
+    { "JS_STATE_PENDING_INITIATE_SENT",
+      TP_ANSI_BOLD_ON                  TP_ANSI_BG_CYAN    },
+    { "JS_STATE_PENDING_INITIATED",
+      TP_ANSI_BOLD_ON                  TP_ANSI_BG_MAGENTA },
+    { "JS_STATE_PENDING_ACCEPT_SENT",
+      TP_ANSI_BOLD_ON                  TP_ANSI_BG_CYAN    },
+    { "JS_STATE_ACTIVE",
+      TP_ANSI_BOLD_ON                  TP_ANSI_BG_BLUE    },
+    { "JS_STATE_ENDED",
+                                       TP_ANSI_BG_RED     }
 };
 
 static void
@@ -211,6 +217,7 @@ create_media_stream (GabbleMediaSession *session,
   gchar *object_path;
   GabbleMediaStream *stream;
   guint id;
+  gboolean h263_n800_hack = FALSE;
 
   g_assert (GABBLE_IS_MEDIA_SESSION (session));
   g_assert (name != NULL);
@@ -240,6 +247,20 @@ create_media_stream (GabbleMediaSession *session,
       media_type == TP_MEDIA_STREAM_TYPE_AUDIO ? "audio" : "video",
       name, id);
 
+  if (media_type == TP_MEDIA_STREAM_TYPE_VIDEO)
+    {
+      GabblePresence *presence = gabble_presence_cache_get (
+        priv->conn->presence_cache, priv->peer);
+
+      if (presence != NULL &&
+          gabble_presence_resource_has_caps (presence, priv->peer_resource,
+            PRESENCE_CAP_JINGLE_H263_N800_HACK))
+        {
+          GMS_DEBUG_INFO (session, "enabling H263-N800 hack");
+          h263_n800_hack = TRUE;
+        }
+    }
+
   object_path = g_strdup_printf ("%s/MediaStream%u", priv->object_path, id);
 
   stream = g_object_new (GABBLE_TYPE_MEDIA_STREAM,
@@ -251,6 +272,7 @@ create_media_stream (GabbleMediaSession *session,
                          "id", id,
                          "initiator", initiator,
                          "media-type", media_type,
+                         "h263-n800-hack", h263_n800_hack,
                          NULL);
 
   g_signal_connect (stream, "notify::connection-state",
@@ -406,7 +428,8 @@ gabble_media_session_class_init (GabbleMediaSessionClass *gabble_media_session_c
   GObjectClass *object_class = G_OBJECT_CLASS (gabble_media_session_class);
   GParamSpec *param_spec;
 
-  g_type_class_add_private (gabble_media_session_class, sizeof (GabbleMediaSessionPrivate));
+  g_type_class_add_private (gabble_media_session_class,
+      sizeof (GabbleMediaSessionPrivate));
 
   object_class->constructor = gabble_media_session_constructor;
 
@@ -426,15 +449,14 @@ gabble_media_session_class_init (GabbleMediaSessionClass *gabble_media_session_c
                                     G_PARAM_STATIC_BLURB);
   g_object_class_install_property (object_class, PROP_CONNECTION, param_spec);
 
-  param_spec = g_param_spec_object ("media-channel", "GabbleMediaChannel object",
-                                    "Gabble media channel object that owns this "
-                                    "media session object.",
-                                    GABBLE_TYPE_MEDIA_CHANNEL,
-                                    G_PARAM_CONSTRUCT_ONLY |
-                                    G_PARAM_READWRITE |
-                                    G_PARAM_STATIC_NICK |
-                                    G_PARAM_STATIC_BLURB);
-  g_object_class_install_property (object_class, PROP_MEDIA_CHANNEL, param_spec);
+  param_spec = g_param_spec_object ("media-channel",
+      "GabbleMediaChannel object",
+      "Gabble media channel object that owns this media session object.",
+      GABBLE_TYPE_MEDIA_CHANNEL,
+      G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_NICK |
+      G_PARAM_STATIC_BLURB);
+  g_object_class_install_property (object_class, PROP_MEDIA_CHANNEL,
+      param_spec);
 
   param_spec = g_param_spec_string ("object-path", "D-Bus object path",
                                     "The D-Bus object path used for this "
@@ -689,8 +711,8 @@ _handle_create (GabbleMediaSession *session,
           else
             {
               g_set_error (error, GABBLE_XMPP_ERROR, XMPP_ERROR_CONFLICT,
-                  "session initiator is creating a stream named \"%s\" already",
-                  stream_name);
+                  "session initiator is creating a stream named \"%s\" "
+                  "already", stream_name);
               return FALSE;
             }
         }
@@ -1397,8 +1419,12 @@ timeout_session (gpointer data)
 
   DEBUG ("session timed out");
 
-  _gabble_media_session_terminate (session, INITIATOR_LOCAL,
-      TP_CHANNEL_GROUP_CHANGE_REASON_ERROR);
+  if (session->initiator == INITIATOR_LOCAL)
+      _gabble_media_session_terminate (session, INITIATOR_LOCAL,
+          TP_CHANNEL_GROUP_CHANGE_REASON_NO_ANSWER);
+  else
+      _gabble_media_session_terminate (session, INITIATOR_LOCAL,
+          TP_CHANNEL_GROUP_CHANGE_REASON_ERROR);
 
   return FALSE;
 }
@@ -1416,8 +1442,8 @@ _add_ready_new_streams (GabbleMediaSession *session)
       GabbleMediaStream *stream = g_ptr_array_index (priv->streams, i);
 
       GMS_DEBUG_DUMP (session, "pondering accept-time add for stream: %s, got "
-          "local codecs: %s, initiator: %s, signalling state: %d", stream->name,
-          stream->got_local_codecs ? "true" : "false",
+          "local codecs: %s, initiator: %s, signalling state: %d",
+          stream->name, stream->got_local_codecs ? "true" : "false",
           stream->initiator == INITIATOR_LOCAL ? "local" : "remote",
           stream->signalling_state);
 
@@ -1462,8 +1488,9 @@ session_state_changed (GabbleMediaSession *session,
       g_source_remove (priv->timer_id);
       priv->timer_id = 0;
 
-      /* signal any streams to the remote end which were added locally & became
-       * ready before the session was accepted, so haven't been mentioned yet */
+      /* signal any streams to the remote end which were added locally &
+       * became ready before the session was accepted, so haven't been
+       * mentioned yet */
       _add_ready_new_streams (session);
     }
 }
@@ -1481,7 +1508,8 @@ _mark_local_streams_sent (GabbleMediaSession *session)
       if (stream->initiator == INITIATOR_REMOTE)
         continue;
 
-      GMS_DEBUG_INFO (session, "marking local stream %s as signalled", stream->name);
+      GMS_DEBUG_INFO (session, "marking local stream %s as signalled",
+          stream->name);
 
       g_object_set (stream, "signalling-state", STREAM_SIG_STATE_SENT, NULL);
     }
@@ -1503,7 +1531,8 @@ _mark_local_streams_acked (GabbleMediaSession *session)
       if (stream->signalling_state != STREAM_SIG_STATE_SENT)
         continue;
 
-      GMS_DEBUG_INFO (session, "marking local stream %s as acknowledged", stream->name);
+      GMS_DEBUG_INFO (session, "marking local stream %s as acknowledged",
+          stream->name);
 
       g_object_set (stream,
           "signalling-state", STREAM_SIG_STATE_ACKNOWLEDGED,
@@ -1524,7 +1553,8 @@ _set_remote_streams_playing (GabbleMediaSession *session)
       if (stream->initiator == INITIATOR_LOCAL)
         continue;
 
-      GMS_DEBUG_INFO (session, "setting remote stream %s as playing", stream->name);
+      GMS_DEBUG_INFO (session, "setting remote stream %s as playing",
+          stream->name);
 
       g_object_set (stream, "playing", TRUE, NULL);
     }
@@ -2061,15 +2091,17 @@ get_jid_for_contact (GabbleMediaSession *session,
   TpBaseConnection *conn;
   const gchar *base_jid;
   TpHandle self;
+  TpHandleRepoIface *contact_handles;
 
   g_assert (GABBLE_IS_MEDIA_SESSION (session));
 
   priv = GABBLE_MEDIA_SESSION_GET_PRIVATE (session);
   conn = (TpBaseConnection *)priv->conn;
+  contact_handles = tp_base_connection_get_handles (conn,
+      TP_HANDLE_TYPE_CONTACT);
   self = conn->self_handle;
 
-  base_jid = tp_handle_inspect (
-      conn->handles[TP_HANDLE_TYPE_CONTACT], handle);
+  base_jid = tp_handle_inspect (contact_handles, handle);
   g_assert (base_jid != NULL);
 
   if (handle == self)
@@ -2197,7 +2229,8 @@ content_remove_msg_reply_cb (GabbleConnection *conn,
   GPtrArray *removing = (GPtrArray *) user_data;
   guint i;
 
-  MSG_REPLY_CB_END_SESSION_IF_NOT_SUCCESSFUL (session, "stream removal failed");
+  MSG_REPLY_CB_END_SESSION_IF_NOT_SUCCESSFUL (session,
+      "stream removal failed");
 
   for (i = 0; i < removing->len; i++)
     destroy_media_stream (session,
@@ -2772,7 +2805,8 @@ direction_msg_reply_cb (GabbleConnection *conn,
   GabbleMediaSession *session = GABBLE_MEDIA_SESSION (user_data);
   GabbleMediaStream *stream = GABBLE_MEDIA_STREAM (object);
 
-  MSG_REPLY_CB_END_SESSION_IF_NOT_SUCCESSFUL (session, "direction change failed");
+  MSG_REPLY_CB_END_SESSION_IF_NOT_SUCCESSFUL (session,
+      "direction change failed");
 
   if (stream->playing)
     {
@@ -2903,7 +2937,8 @@ _gabble_media_session_request_stream_direction (GabbleMediaSession *session,
 static void
 session_handler_iface_init (gpointer g_iface, gpointer iface_data)
 {
-  TpSvcMediaSessionHandlerClass *klass = (TpSvcMediaSessionHandlerClass *)g_iface;
+  TpSvcMediaSessionHandlerClass *klass =
+    (TpSvcMediaSessionHandlerClass *)g_iface;
 
 #define IMPLEMENT(x) tp_svc_media_session_handler_implement_##x (\
     klass, gabble_media_session_##x)
