@@ -17,6 +17,8 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "media-factory.h"
+
 #define DBUS_API_SUBJECT_TO_CHANGE
 
 #include <stdlib.h>
@@ -35,19 +37,21 @@
 #include "debug.h"
 #include "gabble-connection.h"
 #include "gabble-media-channel.h"
-#include "handles.h"
-#include "media-factory.h"
 #include "namespaces.h"
 #include <telepathy-glib/interfaces.h>
 #include "text-mixin.h"
 #include <telepathy-glib/channel-factory-iface.h>
 #include "util.h"
 
-static void gabble_media_factory_iface_init (gpointer g_iface, gpointer iface_data);
-static LmHandlerResult media_factory_jingle_cb (LmMessageHandler*, LmConnection*, LmMessage*, gpointer);
+static void gabble_media_factory_iface_init (gpointer g_iface,
+    gpointer iface_data);
+static LmHandlerResult media_factory_jingle_cb (LmMessageHandler *,
+    LmConnection *, LmMessage *, gpointer);
 
-G_DEFINE_TYPE_WITH_CODE (GabbleMediaFactory, gabble_media_factory, G_TYPE_OBJECT,
-    G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_FACTORY_IFACE, gabble_media_factory_iface_init));
+G_DEFINE_TYPE_WITH_CODE (GabbleMediaFactory, gabble_media_factory,
+    G_TYPE_OBJECT,
+    G_IMPLEMENT_INTERFACE (TP_TYPE_CHANNEL_FACTORY_IFACE,
+      gabble_media_factory_iface_init));
 
 /* properties */
 enum
@@ -76,9 +80,12 @@ struct _GabbleMediaFactoryPrivate
   gboolean dispose_has_run;
 };
 
-#define GABBLE_MEDIA_FACTORY_GET_PRIVATE(o)    (G_TYPE_INSTANCE_GET_PRIVATE ((o), GABBLE_TYPE_MEDIA_FACTORY, GabbleMediaFactoryPrivate))
+#define GABBLE_MEDIA_FACTORY_GET_PRIVATE(o) \
+  (G_TYPE_INSTANCE_GET_PRIVATE ((o), GABBLE_TYPE_MEDIA_FACTORY, \
+                                GabbleMediaFactoryPrivate))
 
-static GObject *gabble_media_factory_constructor (GType type, guint n_props, GObjectConstructParam *props);
+static GObject *gabble_media_factory_constructor (GType type, guint n_props,
+    GObjectConstructParam *props);
 
 static void
 gabble_media_factory_init (GabbleMediaFactory *fac)
@@ -189,7 +196,8 @@ gabble_media_factory_class_init (GabbleMediaFactoryClass *gabble_media_factory_c
   GObjectClass *object_class = G_OBJECT_CLASS (gabble_media_factory_class);
   GParamSpec *param_spec;
 
-  g_type_class_add_private (gabble_media_factory_class, sizeof (GabbleMediaFactoryPrivate));
+  g_type_class_add_private (gabble_media_factory_class,
+      sizeof (GabbleMediaFactoryPrivate));
 
   object_class->constructor = gabble_media_factory_constructor;
   object_class->dispose = gabble_media_factory_dispose;
@@ -209,9 +217,12 @@ gabble_media_factory_class_init (GabbleMediaFactoryClass *gabble_media_factory_c
 
 }
 
-static gboolean _gabble_media_factory_sid_in_use (GabbleMediaFactory *fac, const gchar *sid);
-static GabbleMediaChannel *new_media_channel (GabbleMediaFactory *fac, TpHandle handle);
-static void media_channel_closed_cb (GabbleMediaChannel *chan, gpointer user_data);
+static gboolean _gabble_media_factory_sid_in_use (GabbleMediaFactory *fac,
+    const gchar *sid);
+static GabbleMediaChannel *new_media_channel (GabbleMediaFactory *fac,
+    TpHandle handle);
+static void media_channel_closed_cb (GabbleMediaChannel *chan,
+    gpointer user_data);
 
 /**
  * media_factory_jingle_cb
@@ -228,11 +239,11 @@ media_factory_jingle_cb (LmMessageHandler *handler,
 {
   GabbleMediaFactory *fac = GABBLE_MEDIA_FACTORY (user_data);
   GabbleMediaFactoryPrivate *priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (fac);
-  TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
   LmMessageNode *iq_node, *session_node;
-  const gchar *from, *id, *action, *sid;
-  gchar *resource;
-  TpHandle handle;
+  const gchar *from, *id, *action, *sid, *resource;
+  TpHandle handle = 0;
   GabbleMediaChannel *chan = NULL;
   gboolean chan_is_new = FALSE;
   GError *error = NULL;
@@ -276,14 +287,21 @@ media_factory_jingle_cb (LmMessageHandler *handler,
       goto BAD_REQUEST;
     }
 
-  handle = gabble_handle_for_contact (
-      conn->handles[TP_HANDLE_TYPE_CONTACT], from, FALSE);
+  handle = tp_handle_ensure (contact_repo, from, NULL, NULL);
   if (handle == 0)
     {
       NODE_DEBUG (iq_node, "unable to get handle for sender");
       goto BAD_REQUEST;
     }
 
+  resource = strchr (from, '/');
+  if (resource == NULL || *resource == '\0')
+    {
+      NODE_DEBUG (iq_node, "sender with no resource");
+      goto BAD_REQUEST;
+    }
+  resource++;
+
   id = lm_message_node_get_attribute (iq_node, "id");
   if (id == NULL)
     {
@@ -330,7 +348,6 @@ media_factory_jingle_cb (LmMessageHandler *handler,
 
   DEBUG ("dispatching to session %s", sid);
   g_object_ref (chan);
-  gabble_decode_jid (from, NULL, NULL, &resource);
 
   if (_gabble_media_channel_dispatch_session_action (chan, handle, resource,
         sid, message, session_node, action, &error))
@@ -350,7 +367,8 @@ media_factory_jingle_cb (LmMessageHandler *handler,
     }
 
   g_object_unref (chan);
-  g_free (resource);
+  if (handle)
+    tp_handle_unref (contact_repo, handle);
 
   return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 
@@ -358,6 +376,8 @@ BAD_REQUEST:
   _gabble_connection_send_iq_error (
     priv->conn, message, XMPP_ERROR_BAD_REQUEST, NULL);
 
+  if (handle)
+    tp_handle_unref (contact_repo, handle);
   return LM_HANDLER_RESULT_REMOVE_MESSAGE;
 }
 
@@ -394,7 +414,8 @@ _gabble_media_factory_sid_in_use (GabbleMediaFactory *fac, const gchar *sid)
 }
 
 const gchar *
-_gabble_media_factory_allocate_sid (GabbleMediaFactory *fac, GabbleMediaChannel *chan)
+_gabble_media_factory_allocate_sid (GabbleMediaFactory *fac,
+                                    GabbleMediaChannel *chan)
 {
   const gchar *sid = _gabble_media_factory_get_unique_sid (fac);
 
@@ -460,7 +481,8 @@ media_channel_closed_cb (GabbleMediaChannel *chan, gpointer user_data)
 
   if (priv->session_chans)
     {
-      g_hash_table_foreach_remove (priv->session_chans, _remove_sid_mapping, chan);
+      g_hash_table_foreach_remove (priv->session_chans, _remove_sid_mapping,
+          chan);
     }
 }
 
@@ -503,7 +525,8 @@ new_media_channel (GabbleMediaFactory *fac, TpHandle creator)
 
   if (priv->relay_token != NULL)
     {
-      g_object_set ((GObject *) chan, "gtalk-p2p-relay-token", priv->relay_token, NULL);
+      g_object_set ((GObject *) chan, "gtalk-p2p-relay-token",
+          priv->relay_token, NULL);
     }
 
   DEBUG ("object path %s", object_path);
@@ -527,9 +550,10 @@ jingle_info_send_request (GabbleMediaFactory *fac)
   LmMessageNode *node;
   const gchar *jid;
   GError *error = NULL;
+  TpHandleRepoIface *contact_handles = tp_base_connection_get_handles (base,
+      TP_HANDLE_TYPE_CONTACT);
 
-  jid = tp_handle_inspect (base->handles[TP_HANDLE_TYPE_CONTACT],
-      base->self_handle);
+  jid = tp_handle_inspect (contact_handles, base->self_handle);
   msg = lm_message_new_with_sub_type (jid, LM_MESSAGE_TYPE_IQ,
       LM_MESSAGE_SUB_TYPE_GET);
 
@@ -698,15 +722,16 @@ gabble_media_factory_iface_connecting (TpChannelFactoryIface *iface)
   GabbleMediaFactory *fac = GABBLE_MEDIA_FACTORY (iface);
   GabbleMediaFactoryPrivate *priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (fac);
 
-  g_assert(priv->conn != NULL);
-  g_assert(priv->conn->lmconn != NULL);
+  g_assert (priv->conn != NULL);
+  g_assert (priv->conn->lmconn != NULL);
 
   DEBUG ("adding callbacks");
 
   g_assert (priv->jingle_cb == NULL);
   g_assert (priv->jingle_info_cb == NULL);
 
-  priv->jingle_cb = lm_message_handler_new (media_factory_jingle_cb, fac, NULL);
+  priv->jingle_cb = lm_message_handler_new (media_factory_jingle_cb, fac,
+      NULL);
   lm_connection_register_message_handler (priv->conn->lmconn, priv->jingle_cb,
                                           LM_MESSAGE_TYPE_IQ,
                                           LM_HANDLER_PRIORITY_NORMAL);
@@ -760,20 +785,22 @@ gabble_media_factory_iface_disconnected (TpChannelFactoryIface *iface)
   g_assert (priv->jingle_cb != NULL);
   g_assert (priv->jingle_info_cb != NULL);
 
-  lm_connection_unregister_message_handler (priv->conn->lmconn, priv->jingle_cb,
-                                            LM_MESSAGE_TYPE_IQ);
+  lm_connection_unregister_message_handler (priv->conn->lmconn,
+      priv->jingle_cb, LM_MESSAGE_TYPE_IQ);
   lm_message_handler_unref (priv->jingle_cb);
   priv->jingle_cb = NULL;
 
-  lm_connection_unregister_message_handler (priv->conn->lmconn, priv->jingle_info_cb,
-                                            LM_MESSAGE_TYPE_IQ);
+  lm_connection_unregister_message_handler (priv->conn->lmconn,
+      priv->jingle_info_cb, LM_MESSAGE_TYPE_IQ);
   lm_message_handler_unref (priv->jingle_info_cb);
   priv->jingle_info_cb = NULL;
 
 }
 
 static void
-gabble_media_factory_iface_foreach (TpChannelFactoryIface *iface, TpChannelFunc foreach, gpointer user_data)
+gabble_media_factory_iface_foreach (TpChannelFactoryIface *iface,
+                                    TpChannelFunc foreach,
+                                    gpointer user_data)
 {
   GabbleMediaFactory *fac = GABBLE_MEDIA_FACTORY (iface);
   GabbleMediaFactoryPrivate *priv = GABBLE_MEDIA_FACTORY_GET_PRIVATE (fac);
@@ -781,7 +808,8 @@ gabble_media_factory_iface_foreach (TpChannelFactoryIface *iface, TpChannelFunc
 
   for (i = 0; i < priv->channels->len; i++)
     {
-      foreach (TP_CHANNEL_IFACE (g_ptr_array_index (priv->channels, i)), user_data);
+      foreach (TP_CHANNEL_IFACE (g_ptr_array_index (priv->channels, i)),
+          user_data);
     }
 }
 
@@ -811,8 +839,8 @@ gabble_media_factory_iface_request (TpChannelFactoryIface *iface,
     {
       chan = new_media_channel (fac, conn->self_handle);
 
-      if (!_gabble_media_channel_add_member (
-            TP_SVC_CHANNEL_INTERFACE_GROUP (chan), handle, "", error))
+      if (!_gabble_media_channel_add_member ((GObject *)chan, handle, "",
+            error))
         {
           gabble_media_channel_close (chan);
 
