@@ -18,10 +18,13 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "presence-cache.h"
+
 #include <stdlib.h>
 #include <string.h>
 
-/* when five DIFFERENT guys report the same caps for a given bundle, it'll be enough */
+/* when five DIFFERENT guys report the same caps for a given bundle, it'll
+be enough */
 #define CAPABILITY_BUNDLE_ENOUGH_TRUST 5
 #define DEBUG_FLAG GABBLE_DEBUG_PRESENCE
 
@@ -32,8 +35,6 @@
 #include "namespaces.h"
 #include "util.h"
 
-#include "presence-cache.h"
-
 #include "gabble-signals-marshal.h"
 
 G_DEFINE_TYPE (GabblePresenceCache, gabble_presence_cache, G_TYPE_OBJECT);
@@ -57,7 +58,8 @@ enum
 
 static guint signals[LAST_SIGNAL] = { 0 };
 
-#define GABBLE_PRESENCE_CACHE_PRIV(account) ((GabblePresenceCachePrivate *)account->priv)
+#define GABBLE_PRESENCE_CACHE_PRIV(account) \
+  ((GabblePresenceCachePrivate *)account->priv)
 
 typedef struct _GabblePresenceCachePrivate GabblePresenceCachePrivate;
 
@@ -93,7 +95,10 @@ struct _DiscoWaiter
  * disco_waiter_new ()
  */
 static DiscoWaiter *
-disco_waiter_new (TpHandleRepoIface *repo, TpHandle handle, const gchar *resource, guint serial)
+disco_waiter_new (TpHandleRepoIface *repo,
+                  TpHandle handle,
+                  const gchar *resource,
+                  guint serial)
 {
   DiscoWaiter *waiter;
 
@@ -106,7 +111,8 @@ disco_waiter_new (TpHandleRepoIface *repo, TpHandle handle, const gchar *resourc
   waiter->resource = g_strdup (resource);
   waiter->serial = serial;
 
-  DEBUG ("created waiter %p for handle %u with serial %u", waiter, handle, serial);
+  DEBUG ("created waiter %p for handle %u with serial %u", waiter, handle,
+      serial);
 
   return waiter;
 }
@@ -116,7 +122,8 @@ disco_waiter_free (DiscoWaiter *waiter)
 {
   g_assert (NULL != waiter);
 
-  DEBUG ("freeing waiter %p for handle %u with serial %u", waiter, waiter->handle, waiter->serial);
+  DEBUG ("freeing waiter %p for handle %u with serial %u", waiter,
+      waiter->handle, waiter->serial);
 
   tp_handle_unref (waiter->repo, waiter->handle);
 
@@ -276,7 +283,8 @@ gabble_presence_cache_class_init (GabblePresenceCacheClass *klass)
     G_SIGNAL_RUN_LAST,
     0,
     NULL, NULL,
-    gabble_marshal_VOID__UINT_UINT_UINT, G_TYPE_NONE, 3, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT);
+    gabble_marshal_VOID__UINT_UINT_UINT, G_TYPE_NONE,
+    3, G_TYPE_UINT, G_TYPE_UINT, G_TYPE_UINT);
   signals[AVATAR_UPDATE] = g_signal_new (
     "avatar-update",
     G_TYPE_FROM_CLASS (klass),
@@ -387,16 +395,16 @@ gabble_presence_cache_set_property (GObject     *object,
 {
   GabblePresenceCache *cache = GABBLE_PRESENCE_CACHE (object);
   GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
-  TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
+  TpHandleRepoIface *contact_repo;
   TpHandleSet *new_presence_handles;
 
   switch (property_id) {
     case PROP_CONNECTION:
       priv->conn = g_value_get_object (value);
-      conn = (TpBaseConnection *)priv->conn;
+      contact_repo = tp_base_connection_get_handles (
+          (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
 
-      new_presence_handles = tp_handle_set_new (
-          conn->handles[TP_HANDLE_TYPE_CONTACT]);
+      new_presence_handles = tp_handle_set_new (contact_repo);
 
       if (priv->presence_handles)
         {
@@ -467,8 +475,8 @@ gabble_presence_cache_status_changed_cb (GabbleConnection *conn,
     case TP_CONNECTION_STATUS_CONNECTING:
       g_assert (priv->lm_message_cb == NULL);
 
-      priv->lm_message_cb = lm_message_handler_new (gabble_presence_cache_lm_message_cb,
-                                                    cache, NULL);
+      priv->lm_message_cb = lm_message_handler_new (
+          gabble_presence_cache_lm_message_cb, cache, NULL);
       lm_connection_register_message_handler (priv->conn->lmconn,
                                               priv->lm_message_cb,
                                               LM_MESSAGE_TYPE_PRESENCE,
@@ -481,16 +489,17 @@ gabble_presence_cache_status_changed_cb (GabbleConnection *conn,
     case TP_CONNECTION_STATUS_CONNECTED:
       break;
     case TP_CONNECTION_STATUS_DISCONNECTED:
-      g_assert (priv->lm_message_cb != NULL);
-
-      lm_connection_unregister_message_handler (conn->lmconn,
-                                                priv->lm_message_cb,
-                                                LM_MESSAGE_TYPE_PRESENCE);
-      lm_connection_unregister_message_handler (conn->lmconn,
-                                                priv->lm_message_cb,
-                                                LM_MESSAGE_TYPE_MESSAGE);
-      lm_message_handler_unref (priv->lm_message_cb);
-      priv->lm_message_cb = NULL;
+      if (priv->lm_message_cb != NULL)
+        {
+          lm_connection_unregister_message_handler (conn->lmconn,
+                                                    priv->lm_message_cb,
+                                                    LM_MESSAGE_TYPE_PRESENCE);
+          lm_connection_unregister_message_handler (conn->lmconn,
+                                                    priv->lm_message_cb,
+                                                    LM_MESSAGE_TYPE_MESSAGE);
+          lm_message_handler_unref (priv->lm_message_cb);
+          priv->lm_message_cb = NULL;
+        }
       break;
     default:
       g_assert_not_reached ();
@@ -636,7 +645,8 @@ _grab_avatar_sha1 (GabblePresenceCache *cache,
            * For the moment I'm going to ignore that requirement and
            * trust that our other resource is getting its sha1 right!
            */
-          /* TODO: I don't trust anyone to get XMPP right, so let's do this. :D */
+          /* TODO: I don't trust anyone to get XMPP right, so let's do
+           * this. :D */
         }
 #endif
 
@@ -683,6 +693,30 @@ _extract_cap_bundles (LmMessageNode *lm_node)
   return uris;
 }
 
+/* HACK HACK HACK: Detect Nokis N800 Internet Tablets running Gabble 0.4.x
+ * where x is less than 12, or the Nokia Internet Call Invitation client, which
+ * both connect the H263 codec to a H263+ payloader, creating the unique
+ * H283-N800 variant. Unfortunately they call it H263-1998, so we need to
+ * identify such clients and rewrite between H263-1998 and H263-N800 when
+ * calling them. */
+static GabblePresenceCapabilities
+_detect_h263_n800_hack (const gchar *node)
+{
+  const gchar *version04 = "http://telepathy.freedesktop.org/caps#0.4.";
+  int minor;
+
+  if (strncmp (node, version04, strlen (version04)) != 0)
+    return 0;
+
+  minor = atoi (node + strlen (version04));
+  if (minor == 0 || minor >= 12)
+    return 0;
+
+  DEBUG ("detected old Gabble version, enabling H263-N800 hack");
+
+  return PRESENCE_CAP_JINGLE_H263_N800_HACK;
+}
+
 static void
 _caps_disco_cb (GabbleDisco *disco,
                 GabbleDiscoRequest *request,
@@ -696,15 +730,16 @@ _caps_disco_cb (GabbleDisco *disco,
   LmMessageNode *child;
   GabblePresenceCache *cache;
   GabblePresenceCachePrivate *priv;
-  TpBaseConnection *conn;
+  TpHandleRepoIface *contact_repo;
   gchar *full_jid = NULL;
   GabblePresenceCapabilities caps = 0;
   guint trust;
-  TpHandle handle;
+  TpHandle handle = 0;
 
   cache = GABBLE_PRESENCE_CACHE (user_data);
   priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
-  conn = (TpBaseConnection *)priv->conn;
+  contact_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
 
   if (NULL == node)
     {
@@ -728,13 +763,11 @@ _caps_disco_cb (GabbleDisco *disco,
             {
               const gchar *jid;
 
-              jid = tp_handle_inspect (
-                  conn->handles[TP_HANDLE_TYPE_CONTACT],
-                  waiter->handle);
+              jid = tp_handle_inspect (contact_repo, waiter->handle);
               full_jid = g_strdup_printf ("%s/%s", jid, waiter->resource);
 
-              gabble_disco_request (disco, GABBLE_DISCO_TYPE_INFO, full_jid, node,
-                                    _caps_disco_cb, cache, G_OBJECT(cache), NULL);
+              gabble_disco_request (disco, GABBLE_DISCO_TYPE_INFO, full_jid,
+                  node, _caps_disco_cb, cache, G_OBJECT(cache), NULL);
               waiter->disco_requested = TRUE;
               break;
             }
@@ -742,11 +775,13 @@ _caps_disco_cb (GabbleDisco *disco,
 
       if (NULL != i)
         {
-          DEBUG ("sent a retry disco request to %s for URI %s", full_jid, node);
+          DEBUG ("sent a retry disco request to %s for URI %s", full_jid,
+              node);
         }
       else
         {
-          DEBUG ("failed to find a suitable candidate to retry disco request for URI %s", node);
+          DEBUG ("failed to find a suitable candidate to retry disco "
+              "request for URI %s", node);
           /* FIXME do something very clever here? */
           g_hash_table_remove (priv->disco_pending, node);
         }
@@ -781,8 +816,15 @@ _caps_disco_cb (GabbleDisco *disco,
         caps |= PRESENCE_CAP_CHAT_STATES;
     }
 
-  handle = gabble_handle_for_contact (
-      conn->handles[TP_HANDLE_TYPE_CONTACT], jid, FALSE);
+  caps |= _detect_h263_n800_hack (node);
+
+  handle = tp_handle_ensure (contact_repo, jid, NULL, NULL);
+  if (handle == 0)
+    {
+      DEBUG ("Ignoring presence from invalid JID %s", jid);
+      goto OUT;
+    }
+
   trust = capability_info_recvd (cache, node, handle, caps);
 
   for (i = waiters; NULL != i;)
@@ -837,9 +879,7 @@ _caps_disco_cb (GabbleDisco *disco,
             {
               const gchar *jid;
 
-              jid = tp_handle_inspect (
-                  conn->handles[TP_HANDLE_TYPE_CONTACT],
-                  waiter->handle);
+              jid = tp_handle_inspect (contact_repo, waiter->handle);
               full_jid = g_strdup_printf ("%s/%s", jid, waiter->resource);
 
               gabble_disco_request (disco, GABBLE_DISCO_TYPE_INFO, full_jid,
@@ -864,6 +904,8 @@ _caps_disco_cb (GabbleDisco *disco,
 
 OUT:
 
+  if (handle)
+    tp_handle_unref (contact_repo, handle);
   g_free (full_jid);
 }
 
@@ -877,10 +919,11 @@ _process_caps_uri (GabblePresenceCache *cache,
 {
   CapabilityInfo *info;
   GabblePresenceCachePrivate *priv;
-  TpBaseConnection *conn;
+  TpHandleRepoIface *contact_repo;
 
   priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
-  conn = (TpBaseConnection *)priv->conn;
+  contact_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
   info = capability_info_get (cache, uri, 0);
 
   if (info->trust >= CAPABILITY_BUNDLE_ENOUGH_TRUST
@@ -896,7 +939,8 @@ _process_caps_uri (GabblePresenceCache *cache,
       if (presence)
         {
           GabblePresenceCapabilities save_caps = presence->caps;
-          gabble_presence_set_capabilities (presence, resource, info->caps, serial);
+          gabble_presence_set_capabilities (presence, resource, info->caps,
+              serial);
           g_signal_emit (cache, signals[CAPABILITIES_UPDATE], 0,
               handle, save_caps, presence->caps);
           DEBUG ("caps for %d (%s) now %d", handle, from, presence->caps);
@@ -936,19 +980,19 @@ _process_caps_uri (GabblePresenceCache *cache,
         }
 
       waiters = (GSList *) value;
-      waiter = disco_waiter_new (
-          conn->handles[TP_HANDLE_TYPE_CONTACT], handle,
-          resource, serial);
+      waiter = disco_waiter_new (contact_repo, handle, resource, serial);
       waiters = g_slist_prepend (waiters, waiter);
       g_hash_table_insert (priv->disco_pending, key, waiters);
 
       possible_trust = disco_waiter_list_get_request_count (waiters);
 
-      if (!value || info->trust + possible_trust < CAPABILITY_BUNDLE_ENOUGH_TRUST)
+      if (!value
+          || info->trust + possible_trust < CAPABILITY_BUNDLE_ENOUGH_TRUST)
         {
           /* DISCO */
-          DEBUG ("only %u trust out of %u possible thus far, sending disco for URI %s",
-              info->trust + possible_trust, CAPABILITY_BUNDLE_ENOUGH_TRUST, uri);
+          DEBUG ("only %u trust out of %u possible thus far, sending "
+              "disco for URI %s", info->trust + possible_trust,
+              CAPABILITY_BUNDLE_ENOUGH_TRUST, uri);
           gabble_disco_request (priv->conn->disco, GABBLE_DISCO_TYPE_INFO,
               from, uri, _caps_disco_cb, cache, G_OBJECT (cache), NULL);
           /* enough DISCO for you, buddy */
@@ -963,7 +1007,7 @@ _process_caps (GabblePresenceCache *cache,
                const gchar *from,
                LmMessageNode *lm_node)
 {
-  gchar *resource;
+  const gchar *resource;
   GSList *uris, *i;
   GabblePresenceCachePrivate *priv;
   guint serial;
@@ -971,20 +1015,19 @@ _process_caps (GabblePresenceCache *cache,
   priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
   serial = priv->caps_serial++;
 
-  gabble_decode_jid (from, NULL, NULL, &resource);
-
-  if (NULL == resource)
-    return;
+  resource = strchr (from, '/');
+  if (resource != NULL)
+    resource++;
 
   uris = _extract_cap_bundles (lm_node);
 
   for (i = uris; NULL != i; i = i->next)
     {
-      _process_caps_uri (cache, from, (gchar *) i->data, handle, resource, serial);
+      _process_caps_uri (cache, from, (gchar *) i->data, handle, resource,
+          serial);
       g_free (i->data);
     }
 
-  g_free (resource);
   g_slist_free (uris);
 }
 
@@ -995,8 +1038,7 @@ _parse_presence_message (GabblePresenceCache *cache,
                          LmMessage *message)
 {
   gint8 priority = 0;
-  gchar *resource = NULL;
-  const gchar *status_message = NULL;
+  const gchar *resource, *status_message = NULL;
   LmMessageNode *presence_node, *child_node;
   LmHandlerResult ret = LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
   GabblePresenceId presence_id;
@@ -1005,7 +1047,9 @@ _parse_presence_message (GabblePresenceCache *cache,
   presence_node = message->node;
   g_assert (0 == strcmp (presence_node->name, "presence"));
 
-  gabble_decode_jid (from, NULL, NULL, &resource);
+  resource = strchr (from, '/');
+  if (resource != NULL)
+    resource++;
 
   presence = gabble_presence_cache_get (cache, handle);
 
@@ -1059,8 +1103,6 @@ _parse_presence_message (GabblePresenceCache *cache,
       break;
     }
 
-  g_free (resource);
-
   return ret;
 }
 
@@ -1073,6 +1115,17 @@ _parse_message_message (GabblePresenceCache *cache,
   LmMessageNode *node;
   GabblePresence *presence;
 
+  switch (lm_message_get_sub_type (message))
+    {
+    case LM_MESSAGE_SUB_TYPE_NOT_SET:
+    case LM_MESSAGE_SUB_TYPE_NORMAL:
+    case LM_MESSAGE_SUB_TYPE_CHAT:
+    case LM_MESSAGE_SUB_TYPE_GROUPCHAT:
+      break;
+    default:
+      return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+    }
+
   presence = gabble_presence_cache_get (cache, handle);
 
   if (NULL == presence)
@@ -1084,8 +1137,6 @@ _parse_message_message (GabblePresenceCache *cache,
   node = lm_message_get_node (message);
 
   _grab_nickname (cache, handle, from, node);
-  _grab_avatar_sha1 (cache, handle, from, node);
-  _process_caps (cache, handle, from, node);
 
   return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
 }
@@ -1108,8 +1159,10 @@ gabble_presence_cache_lm_message_cb (LmMessageHandler *handler,
 {
   GabblePresenceCache *cache = GABBLE_PRESENCE_CACHE (user_data);
   GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
-  TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
   const char *from;
+  LmHandlerResult ret;
   TpHandle handle;
 
   g_assert (lmconn == priv->conn->lmconn);
@@ -1122,8 +1175,7 @@ gabble_presence_cache_lm_message_cb (LmMessageHandler *handler,
       return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
     }
 
-  handle = gabble_handle_for_contact (
-      conn->handles[TP_HANDLE_TYPE_CONTACT], from, FALSE);
+  handle = tp_handle_ensure (contact_repo, from, NULL, NULL);
 
   if (0 == handle)
     {
@@ -1134,12 +1186,18 @@ gabble_presence_cache_lm_message_cb (LmMessageHandler *handler,
   switch (lm_message_get_type (message))
     {
     case LM_MESSAGE_TYPE_PRESENCE:
-      return _parse_presence_message (cache, handle, from, message);
+      ret = _parse_presence_message (cache, handle, from, message);
+      break;
     case LM_MESSAGE_TYPE_MESSAGE:
-      return _parse_message_message (cache, handle, from, message);
+      ret = _parse_message_message (cache, handle, from, message);
+      break;
     default:
-      return LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+      ret = LM_HANDLER_RESULT_ALLOW_MORE_HANDLERS;
+      break;
     }
+
+  tp_handle_unref (contact_repo, handle);
+  return ret;
 }
 
 
@@ -1155,10 +1213,10 @@ GabblePresence *
 gabble_presence_cache_get (GabblePresenceCache *cache, TpHandle handle)
 {
   GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
-  TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
 
-  g_assert (tp_handle_is_valid (
-        conn->handles[TP_HANDLE_TYPE_CONTACT], handle, NULL));
+  g_assert (tp_handle_is_valid (contact_repo, handle, NULL));
 
   return g_hash_table_lookup (priv->presence, GINT_TO_POINTER (handle));
 }
@@ -1169,7 +1227,8 @@ gabble_presence_cache_maybe_remove (
     TpHandle handle)
 {
   GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
-  TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
   GabblePresence *presence;
 
   presence = gabble_presence_cache_get (cache, handle);
@@ -1183,8 +1242,7 @@ gabble_presence_cache_maybe_remove (
     {
       const gchar *jid;
 
-      jid = tp_handle_inspect (conn->handles[TP_HANDLE_TYPE_CONTACT],
-          handle);
+      jid = tp_handle_inspect (contact_repo, handle);
       DEBUG ("discarding cached presence for unavailable jid %s", jid);
       g_hash_table_remove (priv->presence, GINT_TO_POINTER (handle));
       tp_handle_set_remove (priv->presence_handles, handle);
@@ -1215,12 +1273,13 @@ gabble_presence_cache_update (
     gint8 priority)
 {
   GabblePresenceCachePrivate *priv = GABBLE_PRESENCE_CACHE_PRIV (cache);
-  TpBaseConnection *conn = (TpBaseConnection *)priv->conn;
+  TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
+      (TpBaseConnection *)priv->conn, TP_HANDLE_TYPE_CONTACT);
   const gchar *jid;
   GabblePresence *presence;
+  GabblePresenceCapabilities caps_before;
 
-  jid = tp_handle_inspect (conn->handles[TP_HANDLE_TYPE_CONTACT],
-      handle);
+  jid = tp_handle_inspect (contact_repo, handle);
   DEBUG ("%s (%d) resource %s prio %d presence %d message \"%s\"",
       jid, handle, resource, priority, presence_id, status_message);
 
@@ -1229,10 +1288,16 @@ gabble_presence_cache_update (
   if (presence == NULL)
     presence = _cache_insert (cache, handle);
 
+  caps_before = presence->caps;
+
   if (gabble_presence_update (presence, resource, presence_id, status_message,
         priority))
     g_signal_emit (cache, signals[PRESENCE_UPDATE], 0, handle);
 
+  if (caps_before != presence->caps)
+    g_signal_emit (cache, signals[CAPABILITIES_UPDATE], 0, handle,
+        caps_before, presence->caps);
+
   gabble_presence_cache_maybe_remove (cache, handle);
 }
 
@@ -1246,3 +1311,30 @@ void gabble_presence_cache_add_bundle_caps (GabblePresenceCache *cache,
   info->caps |= new_caps;
 }
 
+gboolean
+gabble_presence_cache_pep_nick_event_handler (GabbleConnection *conn,
+                                              LmMessage *msg,
+                                              TpHandle handle)
+{
+  const gchar *from;
+  LmMessageNode *node;
+
+  from = lm_message_node_get_attribute (msg->node, "from");
+  if (NULL == from)
+    {
+      NODE_DEBUG (msg->node, "PEP event without from attribute, ignoring");
+      return FALSE;
+    }
+
+  node = lm_message_node_find_child (msg->node, "item");
+  if (NULL == node)
+    {
+    NODE_DEBUG (msg->node, "PEP event without item node, ignoring");
+    return FALSE;
+  }
+
+  _grab_nickname (conn->presence_cache, handle, from, node);
+
+  return TRUE;
+}
+
