@@ -24,11 +24,10 @@
 #include <glib-object.h>
 
 #include "presence.h"
-#include "handles.h"
 
 G_BEGIN_DECLS
 
-#define GABBLE_TYPE_PRESENCE_CACHE gabble_presence_cache_get_type()
+#define GABBLE_TYPE_PRESENCE_CACHE gabble_presence_cache_get_type ()
 
 #define GABBLE_PRESENCE_CACHE(obj) \
   (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
@@ -78,6 +77,8 @@ void gabble_presence_cache_maybe_remove (GabblePresenceCache *cache,
     TpHandle handle);
 void gabble_presence_cache_add_bundle_caps (GabblePresenceCache *cache,
     const gchar *node, GabblePresenceCapabilities caps);
+gboolean gabble_presence_cache_pep_nick_event_handler (GabbleConnection *conn,
+    LmMessage *msg, TpHandle handle);
 
 G_END_DECLS
 
