@@ -29,7 +29,7 @@
 
 G_BEGIN_DECLS
 
-#define GABBLE_TYPE_PRESENCE gabble_presence_get_type()
+#define GABBLE_TYPE_PRESENCE gabble_presence_get_type ()
 
 #define GABBLE_PRESENCE(obj) \
     (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
@@ -58,7 +58,8 @@ typedef enum {
     PRESENCE_CAP_JINGLE = 1 << 2,
     PRESENCE_CAP_JINGLE_DESCRIPTION_AUDIO = 1 << 3,
     PRESENCE_CAP_JINGLE_DESCRIPTION_VIDEO = 1 << 4,
-    PRESENCE_CAP_CHAT_STATES = 1 << 5,
+    PRESENCE_CAP_JINGLE_H263_N800_HACK = 1 << 5,
+    PRESENCE_CAP_CHAT_STATES = 1 << 6,
 } GabblePresenceCapabilities;
 
 struct _GabblePresence {
@@ -82,20 +83,23 @@ GType gabble_presence_get_type (void);
 
 GabblePresence* gabble_presence_new (void);
 
-gboolean gabble_presence_update (GabblePresence *presence, const gchar *resource, GabblePresenceId status, const gchar *status_message, gint8 priority);
+gboolean gabble_presence_update (GabblePresence *presence,
+    const gchar *resource, GabblePresenceId status,
+    const gchar *status_message, gint8 priority);
 
-void gabble_presence_set_capabilities (GabblePresence *presence, const gchar *resource, GabblePresenceCapabilities caps, guint serial);
+void gabble_presence_set_capabilities (GabblePresence *presence,
+    const gchar *resource, GabblePresenceCapabilities caps, guint serial);
 
-const gchar *gabble_presence_pick_resource_by_caps (GabblePresence *presence, GabblePresenceCapabilities caps);
+const gchar *gabble_presence_pick_resource_by_caps (GabblePresence *presence,
+    GabblePresenceCapabilities caps);
 
-gboolean
-gabble_presence_resource_has_caps (GabblePresence *presence,
+gboolean gabble_presence_resource_has_caps (GabblePresence *presence,
                                    const gchar *resource,
                                    GabblePresenceCapabilities caps);
 
-LmMessage *gabble_presence_as_message (GabblePresence *presence, const gchar *resource);
-gchar *
-gabble_presence_dump (GabblePresence *presence);
+LmMessage *gabble_presence_as_message (GabblePresence *presence,
+    const gchar *resource);
+gchar *gabble_presence_dump (GabblePresence *presence);
 
 G_END_DECLS
 
