@@ -23,6 +23,8 @@
 
 #define _GNU_SOURCE /* Needed for strptime (_XOPEN_SOURCE can also be used). */
 
+#include "text-mixin.h"
+
 #include <loudmouth/loudmouth.h>
 #include <dbus/dbus-glib.h>
 #include <string.h>
@@ -40,7 +42,30 @@
 #include "roster.h"
 #include "util.h"
 
-#include "text-mixin.h"
+/**
+ * gabble_text_mixin_init:
+ * @obj_cls: The class of the implementation that uses this mixin
+ * @offset: The offset of the GabbleTextMixinClass within the class structure
+ * @send_nick: %TRUE if the user's nick should be included in messages
+ *             sent through this channel
+ *
+ * Initialize the text mixin. Should be called instead of #tp_text_mixin_init
+ * from the implementation's init function.
+ */
+void
+gabble_text_mixin_init (GObject *obj,
+                        glong offset,
+                        TpHandleRepoIface *contacts_repo,
+                        gboolean send_nick)
+{
+  GabbleTextMixin *mixin;
+
+  tp_text_mixin_init (obj, offset, contacts_repo);
+
+  mixin = GABBLE_TEXT_MIXIN (obj);
+
+  mixin->send_nick = send_nick;
+}
 
 /**
  * gabble_text_mixin_send
@@ -69,13 +94,13 @@ gabble_text_mixin_send (GObject *obj,
                         gboolean emit_signal,
                         GError **error)
 {
-  TpTextMixin *mixin = TP_TEXT_MIXIN (obj);
+  GabbleTextMixin *mixin = GABBLE_TEXT_MIXIN (obj);
   LmMessage *msg;
   LmMessageNode *node;
   gboolean result;
   time_t timestamp;
 
-  if (type > LAST_TP_CHANNEL_TEXT_MESSAGE_TYPE)
+  if (type >= NUM_TP_CHANNEL_TEXT_MESSAGE_TYPES)
     {
       DEBUG ("invalid message type %u", type);
 
@@ -99,7 +124,8 @@ gabble_text_mixin_send (GObject *obj,
         }
     }
 
-  msg = lm_message_new_with_sub_type (recipient, LM_MESSAGE_TYPE_MESSAGE, subtype);
+  msg = lm_message_new_with_sub_type (recipient, LM_MESSAGE_TYPE_MESSAGE,
+      subtype);
 
   if (mixin->send_nick)
     {
@@ -170,15 +196,14 @@ gabble_text_mixin_parse_incoming_message (LmMessage *message,
                         const gchar **from,
                         time_t *stamp,
                         TpChannelTextMessageType *msgtype,
-                        const gchar **body,
-                        const gchar **body_offset,
+                        const gchar **body_ret,
                         gint *state,
                         TpChannelTextSendError *send_error)
 {
-  const gchar *type;
+  const gchar *type, *body;
   LmMessageNode *node;
 
-  *send_error = TP_CHANNEL_SEND_NO_ERROR;
+  *send_error = GABBLE_TEXT_CHANNEL_SEND_NO_ERROR;
 
   if (lm_message_get_sub_type (message) == LM_MESSAGE_SUB_TYPE_ERROR)
     {
@@ -272,11 +297,11 @@ gabble_text_mixin_parse_incoming_message (LmMessage *message,
 
   if (node)
     {
-      *body = lm_message_node_get_value (node);
+      body = lm_message_node_get_value (node);
     }
   else
     {
-      *body = NULL;
+      body = NULL;
     }
 
   /* Messages starting with /me are ACTION messages, and the /me should be
@@ -286,20 +311,20 @@ gabble_text_mixin_parse_incoming_message (LmMessage *message,
    * all other cases. */
 
   *msgtype = TP_CHANNEL_TEXT_MESSAGE_TYPE_NOTICE;
-  *body_offset = *body;
+  *body_ret = body;
 
-  if (*body)
+  if (body != NULL)
     {
-      if (0 == strncmp (*body, "/me ", 4))
+      if (0 == strncmp (body, "/me ", 4))
         {
           *msgtype = TP_CHANNEL_TEXT_MESSAGE_TYPE_ACTION;
-          *body_offset = *body + 4;
+          *body_ret = body + 4;
         }
       else if (type != NULL && (0 == strcmp (type, "chat") ||
                                 0 == strcmp (type, "groupchat")))
         {
           *msgtype = TP_CHANNEL_TEXT_MESSAGE_TYPE_NORMAL;
-          *body_offset = *body;
+          *body_ret = body;
         }
     }
 
@@ -315,7 +340,6 @@ gabble_text_mixin_parse_incoming_message (LmMessage *message,
       return TRUE;
     }
 
-
   node = lm_message_node_get_child_with_namespace  (message->node, "composing",
       NS_CHAT_STATES);
   if (node)
