@@ -1,14 +1,14 @@
 #include <string.h>
 #include <glib.h>
 #include <glib-object.h>
-#include <handles.h>
+#include <gabble-connection.h>
 #include <telepathy-glib/enums.h>
 #include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/errors.h>
 
 void test_handles (guint handle_type)
 {
-  TpHandleRepoIface *repos[LAST_TP_HANDLE_TYPE+1];
+  TpHandleRepoIface *repos[NUM_TP_HANDLE_TYPES];
   TpHandleRepoIface *tp_repo = NULL;
   GError *error = NULL;
   guint i;
@@ -17,11 +17,11 @@ void test_handles (guint handle_type)
   const gchar *jid = "handle.test@foobar";
   const gchar *return_jid;
 
-  for (i = 0; i <= LAST_TP_HANDLE_TYPE; i++)
+  for (i = 0; i < NUM_TP_HANDLE_TYPES; i++)
     {
       repos[i] = NULL;
     }
-  gabble_handle_repos_init (repos);
+  _gabble_connection_create_handle_repos (NULL, repos);
   tp_repo = repos[handle_type];
   g_assert (tp_repo != NULL);
 
@@ -42,23 +42,21 @@ void test_handles (guint handle_type)
   /* Properly return when error out argument isn't provided */
   g_assert (tp_handle_is_valid (tp_repo, 65536, NULL) == FALSE);
 
-  switch (handle_type)
+  if (handle_type == TP_HANDLE_TYPE_LIST)
     {
-      case TP_HANDLE_TYPE_CONTACT:
-          handle = gabble_handle_for_contact (tp_repo, jid, FALSE);
-          break;
-      case TP_HANDLE_TYPE_LIST:
-          jid = "deny";
-          /* fall through */
-      case TP_HANDLE_TYPE_ROOM:
-          handle = tp_handle_request (tp_repo, jid, TRUE);
-          break;
+      /* for the static repo we need a name that's actually valid */
+      jid = "deny";
     }
+  else
+    {
+      /* It's not there to start with, unless we're using the static repo */
+      handle = tp_handle_lookup (tp_repo, jid, NULL, NULL);
+      g_assert (handle == 0);
+    }
+  /* ... but when we call tp_handle_ensure we get a new ref to it */
+  handle = tp_handle_ensure (tp_repo, jid, NULL, NULL);
   g_assert (handle != 0);
 
-  /* Ref it */
-  g_assert (tp_handle_ref (tp_repo, handle) == TRUE);
-
   /* Try to inspect it */
   return_jid = tp_handle_inspect (tp_repo, handle);
   g_assert (!strcmp (return_jid, jid));
@@ -69,31 +67,31 @@ void test_handles (guint handle_type)
       g_assert (tp_handle_client_hold (tp_repo, "TestSuite", handle, NULL) == TRUE);
 
       /* Now unref it */
-      g_assert (tp_handle_unref (tp_repo, handle) == TRUE);
+      tp_handle_unref (tp_repo, handle);
 
       /* Validate it, should be all healthy because client holds it still */
       g_assert (tp_handle_is_valid (tp_repo, handle, NULL) == TRUE);
 
       /* Ref it again */
-      g_assert (tp_handle_ref (tp_repo, handle) == TRUE);
+      tp_handle_ref (tp_repo, handle);
 
       /* Client releases it */
       g_assert (tp_handle_client_release (tp_repo, "TestSuite", handle, NULL) == TRUE);
     }
 
   /* Now unref it */
-  g_assert (tp_handle_unref (tp_repo, handle) == TRUE);
+  tp_handle_unref (tp_repo, handle);
 
   if (handle_type != TP_HANDLE_TYPE_LIST)
     {
       /* Try to unref it again, should fail */
-      g_assert (tp_handle_unref (tp_repo, handle) == FALSE);
+      g_assert (tp_handle_is_valid (tp_repo, handle, NULL) == FALSE);
     }
 
-  for (i = 0; i <= LAST_TP_HANDLE_TYPE; i++)
+  for (i = 0; i < NUM_TP_HANDLE_TYPES; i++)
     {
       if (repos[i])
-        g_object_unref((GObject *)repos[i]);
+        g_object_unref ((GObject *)repos[i]);
     }
 }
 
